/*
 ICC3D - Interactive Color Correction in 3 Dimensions.
 Copyright (C) 2002-2003 Gjvik University College.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package icc3dpackage;


import javax.swing.*;
import java.awt.*;
import java.awt.event.*;


/**
 * This class is used to display information about this application.
 *
 */
public class About extends Thread {

	/**
	 * A string describing the application.
	 */
	private String about[];
	
	/**
	 * A text field containing the current text.
	 */
	private JTextField display;
	
	/**
	 * The about dialog.
	 */
	private JFrame frame;
	
	/**
	 * Used to stop the thread when the user closes the frame.
	 */
	private boolean quit;
	
	/**
	 * Determines whether the thread is active. Only one thread can be active at a time.
	 */
	private static boolean alreadyRunning;

	/**
	 * Creates an about dialog.
	 * Displays about information in a seperate window.
	 *
	 * @param icc3d The main application object.
	 */
	public About(ICC3D icc3d) {
		if (alreadyRunning) {
			return;
		}
		alreadyRunning = true;
		frame = new JFrame("About ICC3D");
		frame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		frame.setIconImage(ICC3D.image);
		display = new JTextField(30);
		display.setEditable(false);
		display.setBackground(Color.lightGray);
		display.setForeground(Color.black);
		display.setFont(new Font("Courier New", Font.BOLD, 12));
		about = new String[] {	"..........:: ICC3D ::..........",
			"Version: " + icc3d.getVersion() + ", 2002-2003",
			"Brougth to you by (in alphabetical order):", "* Anders Rindal *",
			"* Arne Magnus Bakke *", "* Stle Kopperud *", "in collaboration with:",
			"Employer: Dr. Jon Yngve Hardeberg", "and guidance from:", "Dr. Ivar Farup"
		};
		frame.getContentPane().add(createGUI());
		frame.pack();
		frame.setResizable(false);
		frame.setLocation(Toolkit.getDefaultToolkit().getScreenSize().width / 2
				- (frame.getWidth() / 2),
				Toolkit.getDefaultToolkit().getScreenSize().height / 2
				- (frame.getHeight() / 2));
		frame.setVisible(true);
		quit = false;
		this.start();
	}	
	
	/**
	 * Creates the about dialog graphical user interface (GUI).
	 *
	 * @return JPanel with given GUI
	 */
	private JPanel createGUI() {
		JPanel panel = new JPanel();

		panel.setBorder(BorderFactory.createRaisedBevelBorder());
		panel.setLayout(new BorderLayout());
		panel.add(new JLabel(new ImageIcon("icc3dimg/icc3d-logo.jpg")),
				BorderLayout.CENTER);
		JPanel panel2 = new JPanel();

		panel2.setLayout(new BorderLayout());
		JPanel panel3 = new JPanel();

		panel3.setBorder(BorderFactory.createEtchedBorder());
		panel2.add(display, BorderLayout.NORTH);
		JPanel panel4 = new JPanel();

		panel4.setLayout(new FlowLayout());
		JButton ok = new JButton("Ok");

		panel4.add(ok);
		ok.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				quit = true;
			}
		});
		panel2.add(panel4, BorderLayout.SOUTH);
		panel.add(panel2, BorderLayout.SOUTH);
		
		return panel;
	}
	
	/**
	 * Starts the thread execution. Displays text in a textfield, and loops until canceled.
	 */
	public void run() {
		int i, aboutIndex = 0, partLen = 0;
		long sleep;
		String displayText = new String("");
		
		for (;;) {
			if (aboutIndex == about.length) {
				aboutIndex = 0;
			}
			displayText = about[aboutIndex];
			partLen = about[aboutIndex].length();
			aboutIndex++;	
			try {
				Thread.currentThread().sleep(250);
				sleep = 20;
				for (i = 0; i <= partLen; i++) {
					if (quit) {
						alreadyRunning = false;
						frame.dispose();
						return;
					}
					display.setText(displayText.substring(0, i));
					Thread.currentThread().sleep(sleep);
					sleep += (150 / partLen);
				}
				Thread.currentThread().sleep(2000);
				sleep = 150;
				for (i = partLen; i >= 0; i--) {
					if (quit) {
						alreadyRunning = false;
						frame.dispose();
						return;
					}
					display.setText(displayText.substring(0, i));
					Thread.currentThread().sleep(sleep);
					sleep -= (150 / partLen);
				}
			} catch (InterruptedException ie) {}
		}
	}
}
