/*
 ICC3D - Interactive Color Correction in 3 Dimensions.
 Copyright (C) 2002-2003 Gjvik University College.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package icc3dpackage;


import javax.swing.*;
import java.awt.*;
import java.awt.event.*;


/**
 * A customized frame that acts as a container for all the parameter panels in
 * the application.
 */

public class ControlArea extends JFrame {

	/** The panel that contains all components. */
	private JPanel currentPanel;	
	
	/**
	 * Constructor that initializes the frame, sets its title and size
	 */
	public ControlArea() {
		super("Control panel");
		setIconImage(ICC3D.image);
		setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		setSize(295, 600);
		setLocation(0, 310);
		getContentPane().setLayout(new BorderLayout());
	}

	/**
	 * Function that is invoked each time you change your 3d view. It receives a 
	 * new adjustment panel, which is added to the frame and shown. The function
	 * also ensures that the frame is actually repainted.
	 * @param adjustmentPanel the new panel.
	 */
	public void setControl(JPanel adjustmentPanel) {
		if (currentPanel != adjustmentPanel) {
			currentPanel = adjustmentPanel;
			getContentPane().removeAll();
			getContentPane().add(adjustmentPanel, BorderLayout.CENTER);
			adjustmentPanel.invalidate();
			getContentPane().doLayout();
			getContentPane().validate();
			repaint();
		}
	}
	
	/**
	 * Checks if the given panel is active at this time, and if so replaces it
	 * with an empty panel.
	 * @param oldPanel The old adjustment panel.
	 */
	public void invalidateControl(JPanel oldPanel) {
		if (currentPanel == oldPanel) {
			JPanel tmpPanel = new JPanel();

			tmpPanel.invalidate();
			getContentPane().removeAll();
			getContentPane().add(tmpPanel, BorderLayout.CENTER);
			getContentPane().doLayout();
			getContentPane().validate();
			this.setTitle("Control panel");
		}
	}
} 

