/*
 ICC3D - Interactive Color Correction in 3 Dimensions.
 Copyright (C) 2002-2003 Gjvik University College.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package icc3dpackage;


import java.awt.*;
import javax.swing.*;
import java.awt.color.*;

import icc3dpackage.managers.*;


/**
 * This class is used to display a progress bar for time consuming operations.
 *
 */
public class ProgressBar extends JFrame {
	
	/**
	 * The number of steps.
	 */
	private int steps;
	
	/**
	 * The progress so far.
	 */
	private int progress;
	
	/**
	 * The progress panel.
	 */
	private JPanel progressArea, all;
	
	/**
	 * Description of the action in progress.
	 */
	private String description;
	
	/**
	 * The entire progress bar area.
	 */
	private JPanel bigPanel;
	
	/**
	 * A label used when drawing an hour glass.
	 */
	private JLabel hourGlass;
	
	/**
	 * The hour glass state.
	 */
	private int time;
	
	/**
	 * Default constructor.
	 * Creates a progress bar that moves from start to end with the desired title.
	 * @param steps The number of steps.
	 * @param title The progress bar title.
	 * @param description The initial progress bar description.
	 */
	public ProgressBar(int steps, String title, String description) {
		this.setSize(510, 150);
		this.setTitle(title);
		this.setIconImage(ICC3D.image);
		this.steps = steps;
		this.description = description;
		this.setLocation(Toolkit.getDefaultToolkit().getScreenSize().width / 2
				- this.getWidth() / 2,
				Toolkit.getDefaultToolkit().getScreenSize().height / 2
				- this.getHeight() / 2);
		createGui();
		this.setResizable(false);
		this.setVisible(true);
		paintAll(this.getGraphics());
		doUpdate();
	}
	
	/**
	 * Forces an update of the progress bar.
	 */
	public void doUpdate() {
		Dimension dim = bigPanel.getSize();

		time++;
		if (time == 3) {
			time = 1;
		}
		hourGlass.setIcon(IconManager.getIcon("time" + time));
		bigPanel.paintImmediately(0, 0, (int) dim.getWidth(), (int) dim.getHeight());
	}
	
	/**
	 * Creates the GUI.
	 */
	private void createGui() {
		JPanel tmpPanel = new JPanel();

		tmpPanel.setLayout(new BorderLayout());
		tmpPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createLoweredBevelBorder(),
				"Current progress"));
		progressArea = new JPanel() {
			public void paint(Graphics g) {
				if (progress > 0 && progress <= steps) {
					g.fillRect(0, 0,
							(int) ((progressArea.getWidth()) * progress / (float) steps), 10);
				}
			}
		};
		
		JPanel bevelPanel = new JPanel(new BorderLayout());

		bevelPanel.setBorder(BorderFactory.createRaisedBevelBorder());
		bevelPanel.add(progressArea);
		all = new JPanel() {
			public void paint(Graphics g) {
				g.drawString(description, 10, 40);
				g.drawString("| start [" + 0 + "]", 0, 20);
				g.drawString("[" + steps + "] end |", all.getWidth() - 43, 20);
			}
		};
		all.setBorder(BorderFactory.createRaisedBevelBorder());
		tmpPanel.add(bevelPanel, BorderLayout.NORTH);
		tmpPanel.add(all, BorderLayout.CENTER);
		hourGlass = new JLabel(IconManager.getIcon("time1"));
		
		JPanel south = new JPanel();

		south.setLayout(new BorderLayout());
		south.add(new JLabel(IconManager.getIcon("constraint_free_movement")),
				BorderLayout.WEST);
		bigPanel = new JPanel(new BorderLayout());
		bigPanel.add(tmpPanel, BorderLayout.CENTER);
		bigPanel.add(hourGlass, BorderLayout.EAST);
		bigPanel.add(south, BorderLayout.SOUTH);	
		
		Container c = this.getContentPane();

		c.setLayout(new BorderLayout());
		c.add(bigPanel, BorderLayout.CENTER);
	}
	
	/**
	 * Increase this progress bar by one step.
	 */
	public void inc() {
		progress++;
		doUpdate();
	}
	
	/**
	 * Increase this progress bar by one step and display descriptive text.
	 * @param description descriptive label.
	 */
	public void inc(String description) {
		this.description = description;
		inc();
	}
	
	/**
	 * Set description.
	 * @param description description
	 */
	public void setDescription(String description) {
		this.description = description;
		doUpdate();
	}
}
