/*
 ICC3D - Interactive Color Correction in 3 Dimensions.
 Copyright (C) 2002-2003 Gjvik University College.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package icc3dpackage;


import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import javax.swing.*;
import javax.media.jai.*;
import java.awt.image.renderable.ParameterBlock;

import icc3dpackage.image2d.*;


/**
 * This class is used to display various information(which plugins are loaded etc.)
 * while the application is starting.
 */
public class Splash extends JFrame implements Runnable {
	
	/** The main application. */
	private ICC3D icc3d;
	
	/** An ImageCanvas2D used for image manipulations.	*/
	private ImageCanvas2D img2D;
	
	/** A planar image representing the image.	*/
	private PlanarImage planarImage;
	
	/** A panel that contains status information. */
	private JPanel status;
	
	/** Set exit = true to stop the splash thread. */
	private boolean exit = false;
	
	/** Outputarea for text to display. */
	public JTextArea statusText;
	
	/** A panel that contains all splash components. */
	private JPanel container;
	 
	/**
	 * Default constructor. Creates a splash screen.
	 *
	 * @param icc3d ICC3D object
	 */
	public Splash(ICC3D icc3d) {
		this.icc3d = icc3d;
		setTitle("ICC3D loading");
		setIconImage(ICC3D.image);
		setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		container = new JPanel(new BorderLayout());
		getContentPane().setLayout(new BorderLayout());
		getContentPane().add(container, BorderLayout.CENTER);
		planarImage = FileReader.readImage("icc3dimg" + java.io.File.separator
				+ "icc3d-logo.jpg");
		img2D = new ImageCanvas2D(planarImage);
		img2D.setBorder(BorderFactory.createRaisedBevelBorder());
		img2D.addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent e) {
				checkForTriggerEvent(e);
			}

			public void mouseReleased(MouseEvent e) {
				checkForTriggerEvent(e);
			}

			public void checkForTriggerEvent(MouseEvent e) {
				exit = true;
			}
		});
		container.add(img2D, BorderLayout.CENTER);
		statusText = new JTextArea("ICC3D " + icc3d.getVersion()
				+ "\n\n",
				4,
				50);
		statusText.setLineWrap(true);
		statusText.setBackground(container.getBackground());
		container.add(new JScrollPane(statusText, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
				JScrollPane.HORIZONTAL_SCROLLBAR_NEVER),
				BorderLayout.SOUTH);
		setSize(planarImage.getWidth() + 5, planarImage.getHeight() + 95);
		setLocation(Toolkit.getDefaultToolkit().getScreenSize().width / 2
				- (getWidth() / 2),
				Toolkit.getDefaultToolkit().getScreenSize().height / 2 - (getHeight() / 2));
		setResizable(false);
		setVisible(true);
		Dimension dim = container.getSize();
		container.paintImmediately(0, 0, (int) dim.getWidth(), (int) dim.getHeight());
	}
	
	/**
	 * This thread updates the splash screen continually while the application
	 * is searching for and dynamically loading plugins.
	 */
	public void run() {
		boolean doneIt = false;
		int i;

		try {
			PlanarImage invert = JAI.create("invert", img2D.getPlanarImage());

			for (i = 0; i < 20; i++) {
				if (exit) {
					break;
				}
				Thread.currentThread().sleep(100);
			}
			for (i = 0; i < 20; i++) {
				if (exit) {
					break;
				}
				img2D.addConst(-13, -13, -13);
				Thread.currentThread().sleep(20);	
			}
			if (!exit) {
				img2D.updateImage(invert);
				statusText.append("\nInitializing ...");
				statusText.setCaretPosition(statusText.getText().length());
				Thread.currentThread().sleep(3000);
			}
			for (i = 0; i < 85; i++) {
				if (exit) {
					break;
				}
				img2D.addConst(3, 3, 3);
				Thread.currentThread().sleep(30);
			}	
			for (i = 0; i < 10; i++) {
				if (exit) {
					break;
				}
				Thread.currentThread().sleep(100);
			}
		} catch (InterruptedException e) {
			e.printStackTrace();	
		}
		finally {
			icc3d.setVisible(true);
			Splash.this.setVisible(false);
		}
	}

}
