/*
 ICC3D - Interactive Color Correction in 3 Dimensions.
 Copyright (C) 2002-2003 Gjvik University College.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package icc3dpackage.constraints;


import javax.swing.*;
import javax.vecmath.*;

import icc3dpackage.*;
import icc3dpackage.constraints.generic.*;
import icc3dpackage.managers.*;
import icc3dpackage.pickable.*;


/**
 * Limits the possible movement to a constant angle and height.
 */
public class ConstantAngleAndHeightConstraint extends LineConstraint {
	
	/**
	 * Constructor that initializes the object
	 */
	public ConstantAngleAndHeightConstraint() {}
	
	/**
	 * Returns the end point, it calculates the new end point given the wanted movement and the constant 
	 * height and angle restrictions
	 * @return float[] the end coordinates for the move
	 */	
	public float[] getRealEndPoint(float start[], float stop[], PickableObject objects[]) {
		linePoint = new Vector3f(0.5f, start[1], 0.5f);
		return super.getRealEndPoint(start, stop, objects);
	}

	/**
	 * Returns the icons for this constraint, both active and inactive
	 * @return Icon[] the icons
	 */
	public Icon[] getIcon() {
		Icon icons[] = new Icon[2];

		icons[0] = IconManager.getIcon("constraint_hl");
		icons[1] = IconManager.getIcon("constraint_hl_sel");
		return icons;
	}

	/**
	 * Returns the name of the class
	 * @return String name
	 */
	public String getName() {
		return "Constant hue and lightness";
	}
	
	/**
	 * The priority of the constraint.
	 * @return An int representing the priority of the constraint (lower => higher priority).
	 */
	public int getPriority() {
		return 4;
	}
}
