/*
 ICC3D - Interactive Color Correction in 3 Dimensions.
 Copyright (C) 2002-2003 Gjvik University College.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package icc3dpackage.constraints;


import javax.swing.*;
import javax.vecmath.*;

import icc3dpackage.*;
import icc3dpackage.constraints.generic.*;
import icc3dpackage.managers.*;
import icc3dpackage.pickable.*;
import icc3dpackage.triangulation.*;


/**
 * Limits the movement for the objects within a constant angle.
 */
public class ConstantAngleConstraint extends PlaneConstraint {

	/**
	 * Returns the end point, it calculates the new end point given the wanted movement and plane restrictions
	 * @return float[] the end coordinates for the move.
	 */
	public float[] getRealEndPoint(float start[], float stop[], PickableObject objects[]) {
		Vector3f vec1 = new Vector3f(0.5f, 0.5f, 0.5f);
		
		Vector3f vec2 = new Vector3f(start[0], 0.0f, start[2]);
		Vector3f vec3 = new Vector3f(start[0], 1.0f, start[2]);

		plane = new Plane(vec1, vec3, vec2);
		return super.getRealEndPoint(start, stop, objects);	
	}
	
	/**
	 * Returns the icons for this constraint (both active and inactive).
	 * @return Icon[] the icons.
	 */
	public Icon[] getIcon() {
		Icon icons[] = new Icon[2];

		icons[0] = IconManager.getIcon("constraint_h");
		icons[1] = IconManager.getIcon("constraint_h_sel");
		return icons;
	}
	
	/**
	 * Returnes the name of this class.
	 * @return String name.
	 */
	public String getName() {
		return "Constant hue";
	}
	
	/**
	 * The priority of the constraint.
	 * @return An int representing the priority of the constraint (lower => higher priority).
	 */
	public int getPriority() {
		return 2;
	}
}
