/*
 ICC3D - Interactive Color Correction in 3 Dimensions.
 Copyright (C) 2002-2003 Gjvik University College.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package icc3dpackage.constraints.generic;


import java.awt.*;
import javax.swing.*;
import javax.vecmath.*;

import icc3dpackage.*;
import icc3dpackage.constraints.generic.*;
import icc3dpackage.pickable.*;


/**
 * Base class for other classes that define constraints on lines.
 */
public abstract class LineConstraint extends InteractionConstraint {
	
	/** The end given point for this line. */
	protected Vector3f linePoint;
	
	/**
	 * Constructor that initializes the object
	 */
	public LineConstraint() {
		linePoint = new Vector3f(0.5f, 0.5f, 0.5f);
	}
	
	/**
	 * Returns the end point, it calculates the new end points given the wanted movement and the restrictions
	 * @return float[] the end coordinates for the move
	 */
	public float[] getRealEndPoint(float start[], float stop[], PickableObject objects[]) {
		Vector3f sPoint = new Vector3f(start);
		Vector3f ePoint = new Vector3f(stop);
		
		Vector3f limitedDirection = new Vector3f(start);

		limitedDirection.sub(linePoint);
		limitedDirection.normalize();
		
		Vector3f wantedDirection = new Vector3f(stop);

		wantedDirection.sub(sPoint);
		
		float length = wantedDirection.dot(limitedDirection);

		limitedDirection.scale(length);
		Vector3f retPoint = new Vector3f(limitedDirection);

		retPoint.add(sPoint);
		
		float retFloat[] = new float[3];

		retPoint.get(retFloat);
		
		return retFloat;
		
	}
}
