/*
 ICC3D - Interactive Color Correction in 3 Dimensions.
 Copyright (C) 2002-2003 Gjvik University College.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package icc3dpackage.constraints.generic;


import java.awt.*;
import javax.swing.*;
import javax.vecmath.*;

import icc3dpackage.*;
import icc3dpackage.pickable.*;
import icc3dpackage.triangulation.*;


/**
 * Base class for other classes that define constraints as movements in planes.
 */
public abstract class PlaneConstraint extends InteractionConstraint {

	/**
	 * The plane that defines the limit of this constraint.
	 */
	protected Plane plane;
	
	/**
	 * Constructor that initializes the object (creates a new plane)
	 */	
	public PlaneConstraint() {
		plane = new Plane(new Vector3f(1.0f, 0.0f, 0.0f),
				new Vector3f(0.0f, 0.0f, 0.0f), new Vector3f(1.0f, 0.0f, 1.0f));
	}
	
	/**
	 * Returns the end point, it calculates the new end points given the wanted movement and the restrictions
	 * @return float[] the end coordinates for the move
	 */
	public float[] getRealEndPoint(float start[], float stop[], PickableObject objects[]) {
		Point3f actualStop = plane.projectPointToPlane(stop[0], stop[1], stop[2]);
		float f[] = new float[3];

		actualStop.get(f);
		return f;
	}
}
