/*
 ICC3D - Interactive Color Correction in 3 Dimensions.
 Copyright (C) 2002-2003 Gjvik University College.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package icc3dpackage.image2d;


import java.io.File;
import javax.swing.filechooser.FileFilter;


/**
 * A class used to filter out irrelevant files in a JFileChooser dialog.
 */
public class CustomFileFilter extends FileFilter {
	
	/** A description of the file type. */
	private String description;
	
	/** A description of the file type and its file extensions. */
	private String fullDescription;
	
	/** Display all extensions in the description. */
	private boolean showAllExtensions;
	
	/** A list of file extensions associated by this file type. */
	private String extension[];
	
	/**
	 * Creates a custom file filter with the given description and extensions.
	 * @param description A description of the file type.
	 * @param showAllExtensions Display all extensions in the description.
	 * @param extension The file extensions.
	 */
	public CustomFileFilter(String description, boolean showAllExtensions, String extension[]) {
		this.description = description;
		this.showAllExtensions = showAllExtensions;
		this.extension = extension;
	}
	
	/**
	 * @return true if the file should be displayed in the JFileChooser, otherwise false.
	 */
	public boolean accept(File f) {
		if (f.isDirectory()) {
			return true;
		}
		String name = f.getName();
		int lastIndex = name.lastIndexOf('.');

		if (lastIndex < 0 || lastIndex > name.length() - 1) {
			return false;
		}
		String ext = name.substring(lastIndex + 1);

		for (int i = 0; i < extension.length; i++) {
			if (ext.equalsIgnoreCase(extension[i])) {
				return true;
			}
		}
		return false;
	}
	
	/**
	 * Returns a description of the file type and its file extensions.
	 */
	public String getDescription() {
		if (fullDescription != null) {
			return fullDescription;
		}
		fullDescription = (description == null) ? "" : description;
		if (!showAllExtensions) {
			return fullDescription;
		}
		fullDescription += " (";
		for (int i = 0; i < extension.length - 1; i++) {
			fullDescription += "*." + extension[i] + ", ";
		}
		fullDescription += "*." + extension[extension.length - 1] + ")";
		return fullDescription;
	}
	
	/**
	 * Returns the first extension associated with this filter.
	 * @return The first extension.
	 */
	public String getExtension() {
		if (extension == null || extension[0] == null) {
			return new String();
		}
		return extension[0];
	}
}
