/*
 ICC3D - Interactive Color Correction in 3 Dimensions.
 Copyright (C) 2002-2003 Gjvik University College.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package icc3dpackage.image2d;


import java.awt.image.IndexColorModel;
import java.awt.image.RenderedImage;
import javax.swing.JOptionPane;
import java.io.*;
import javax.media.jai.JAI;
import javax.media.jai.LookupTableJAI;
import javax.media.jai.PlanarImage;
import com.sun.media.jai.codec.ImageCodec;
import com.sun.media.jai.codec.ImageDecoder;
import com.sun.media.jai.codec.FileSeekableStream;
import com.sun.media.jai.codec.SeekableStream;


/**
 * This class is used to load imagefiles, and return them as PlanarImages.
 *
 */
public abstract class FileReader {
	
	/**
	 * Reads an image file, and constructs a PlanarImage.
	 *	
	 * For example: the following code will try to read a file from present working directory:
	 * PlanarImage planarImage = FileReader.readImage("foo.tiff");
	 *
	 * @param fileName absolute or relative path to file. For example: [path/]foo.tif
	 * @return image as a PlanarImage if success, otherwise null
	 * @see javax.media.jai.PlanarImage
	 */ 
	public static PlanarImage readImage(String fileName) {
		PlanarImage image = null;
		
		try {
			if (System.getProperty("JAI_IMAGE_READER_USE_CODECS") == null) {
				image = JAI.create("fileload", fileName).createInstance();
			} else {
				try {
					SeekableStream stream = new FileSeekableStream(fileName);
					String[] names = ImageCodec.getDecoderNames(stream);
					ImageDecoder dec = ImageCodec.createImageDecoder(names[0], stream, null);
					RenderedImage ri = dec.decodeAsRenderedImage();

					image = PlanarImage.wrapRenderedImage(ri);
					stream.close();
				} catch (Exception e) {
					image = null;
				}
			}
    	
			if (image != null && image.getColorModel() instanceof IndexColorModel) {
				IndexColorModel icm = (IndexColorModel) image.getColorModel();
				int mapSize = icm.getMapSize();
				byte[][] lutData = new byte[3][mapSize];

				icm.getReds(lutData[0]);
				icm.getGreens(lutData[1]);
				icm.getBlues(lutData[2]);
				LookupTableJAI lut = new LookupTableJAI(lutData);

				image = JAI.create("lookup", image, lut);
			}

		} catch (IllegalArgumentException e) {
			image = null;
		} catch (Exception e) {
			image = null;
		}
		return image;
	}
}
