/*
 ICC3D - Interactive Color Correction in 3 Dimensions.
 Copyright (C) 2002-2003 Gjvik University College.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package icc3dpackage.image2d;


import java.io.*;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.lang.*;
import javax.media.j3d.Canvas3D;
import javax.media.jai.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.filechooser.FileFilter;
import java.util.*;
import javax.swing.border.*;
import java.awt.color.ColorSpace;

import icc3dpackage.*;
import icc3dpackage.colorspaces.generic.ColorSpace3D;
import icc3dpackage.managers.*;
import icc3dpackage.modules.*;
import icc3dpackage.tiffreader.TIFF_Writer;


/**
 * This class is used to represent a session. It displays an image and options to perform.
 */
public class Image2D extends JFrame implements ActionListener {
	
	/** The main application */
	private ICC3D root;
	
	/** The session id */
	private int sessionId;
		
	/** The file chooser used for saving the image */
	private JFileChooser save;
	
	/** The color space assigned to the monitor */
	private JComboBox viewSpace;
	
	/** GUI button */
	private JButton saveAs,
			saveTIFF,
			undo,
			addConst,
			show3D,
			closeSession,
			imageInfo;
	
	/** The 2D image */
	private ImageCanvas2D imageCanvas2D;
	
	/** The thumbnail image */
	private ImageCanvas2D_Thumbnail thumbnail;
	
	/** All associated image modules */
	private Vector imageModules;
	
	/**
	 * Constructs a 2d image.
	 * @param icc3d The main application.
	 * @param id The image id.
	 * @param p The Image selected in the open dialog.
	 */
	public Image2D(ICC3D icc3d, int id, ImageOpenParameter p) {
		root = icc3d;
		this.sessionId = id;
		imageModules = new Vector();

		setTitle("Image " + sessionId);
		setIconImage(ICC3D.image);
		imageCanvas2D = new ImageCanvas2D(p);
		thumbnail = imageCanvas2D.createThumbnail(180, 180);
    
		JPanel panel = new JPanel();

		panel.setLayout(new BorderLayout(2, 2));
				
		JPanel imgPanel = new JPanel();

		imgPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createLoweredBevelBorder(),
				"Image"));
		imgPanel.setLayout(new BorderLayout());
		imgPanel.add(new JScrollPane(imageCanvas2D), BorderLayout.CENTER);
		panel.add(imgPanel, BorderLayout.CENTER);		
		
		JPanel p2 = new JPanel();

		p2.setLayout(new BorderLayout());
		
		JPanel p3 = new JPanel();

		p3.setLayout(new GridLayout(0, 1));
		p3.setBorder(BorderFactory.createTitledBorder(BorderFactory.createRaisedBevelBorder(),
				"Image options"));
		imageInfo = new JButton("Image info", IconManager.getIcon("binary_question"));
		imageInfo.setEnabled(false);
		imageInfo.setHorizontalAlignment(JButton.LEFT);
		imageInfo.addActionListener(this);
		p3.add(imageInfo);
		saveAs = new JButton("Save as", IconManager.getIcon("save"));
		saveAs.setHorizontalAlignment(JButton.LEFT);
		saveAs.addActionListener(this);
		p3.add(saveAs);
		saveTIFF = new JButton("Save TIFF", IconManager.getIcon("save"));
		saveTIFF.setHorizontalAlignment(JButton.LEFT);
		saveTIFF.addActionListener(this);
		p3.add(saveTIFF);
		undo = new JButton("Revert to original", IconManager.getIcon("undo"));
		undo.setHorizontalAlignment(JButton.LEFT);
		undo.addActionListener(this);
		p3.add(undo);
		addConst = new JButton("Add constant", IconManager.getIcon("addconst"));
		addConst.setEnabled(false);
		addConst.setHorizontalAlignment(JButton.LEFT);
		addConst.addActionListener(this);
		p3.add(addConst);
		show3D = new JButton("View 3D", IconManager.getIcon("show3d"));
		show3D.setHorizontalAlignment(JButton.LEFT);
		show3D.addActionListener(this);
		p3.add(show3D);
		closeSession = new JButton("Close image", IconManager.getIcon("exit"));
		closeSession.setHorizontalAlignment(JButton.LEFT);
		closeSession.addActionListener(this);

		viewSpace = new JComboBox(ColorSpaceManager.getNames());
		for (int i = 0; i < viewSpace.getItemCount(); i++) {
			if (viewSpace.getItemAt(i).equals("sRGB")) {
				viewSpace.setSelectedIndex(i);
			}
		}
		viewSpace.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				if (e.getStateChange() == ItemEvent.SELECTED) {
					imageCanvas2D.setViewColorSpace(ColorSpaceManager.getColorSpace(viewSpace.getSelectedItem().toString()));
					Arrays.fill(imageCanvas2D.getDirtyColors(), true);
					imageCanvas2D.imageUpdate();
				}
			}
		});
		JPanel spacePanel = new JPanel(new BorderLayout());

		spacePanel.add(p3, BorderLayout.CENTER);
		spacePanel.add(viewSpace, BorderLayout.SOUTH);
		JPanel tmp = new JPanel();

		tmp.add(closeSession);
		p2.add(tmp, BorderLayout.SOUTH);
		p2.add(thumbnail, BorderLayout.CENTER);
		p2.add(spacePanel, BorderLayout.NORTH);
		panel.add(p2, BorderLayout.WEST);
		getContentPane().add(panel);
		
		createSaveDialog();
		this.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				root.getSessionManager().removeImage2D(Image2D.this);
			}
		});
		setLocation(300, 0);
		setSize(700, 500);
		setVisible(true);
	}
	
	/**
	 * Creates a JFileChooser savedialog for subsequent use. Default filter is "tiff".
	 *	
	 * @see javax.swing.JFileChooser
	 */
	private void createSaveDialog() {
		save = new JFileChooser(new File("."));
		save.setFileSelectionMode(JFileChooser.FILES_ONLY);
		save.setFileFilter(new CustomFileFilter("Bitmap Image", true,
				new String[] {"bmp"}));
		save.addChoosableFileFilter(new CustomFileFilter("Joint Photographic Experts Group",
				true, new String[] {"jpg"}));
		save.addChoosableFileFilter(new CustomFileFilter("Portable Network Graphics",
				true, new String[] {"png"}));
		save.addChoosableFileFilter(new CustomFileFilter("PNM file", true,
				new String[] {"pnm"}));
		save.addChoosableFileFilter(new CustomFileFilter("Tagged Image Format", true,
				new String[] {"tif"}));
	}
	
	/**
	 * Invokes necessary routines for a given event.
	 *	
	 * @param ae event
	 */
	public void actionPerformed(ActionEvent ae) {
		if (ae.getSource() == saveTIFF) {
			TIFF_Writer.saveImage(imageCanvas2D);
		} else if (ae.getSource() == saveAs) {
			if (save.showSaveDialog(ICC3D.getDialogParent()) != JFileChooser.CANCEL_OPTION) {
				String extension, tmp;
				String fileName = new String(save.getSelectedFile().getAbsolutePath());
				FileFilter tmpFilter = save.getFileFilter();

				if (tmpFilter instanceof CustomFileFilter) {
					CustomFileFilter filter = (CustomFileFilter) tmpFilter;

					extension = filter.getExtension();
					tmp = fileName.substring(fileName.lastIndexOf(".") + 1, fileName.length());
					if (tmp == null || !tmp.equalsIgnoreCase(extension)) {
						fileName += "." + extension;
					}
					FileWriter.writeImage(imageCanvas2D.getPlanarImage(), fileName);
					fileName = null;
				} else {
					FileWriter.writeImage(imageCanvas2D.getPlanarImage(), fileName);
				}
			}
		} else if (ae.getSource() == undo) { 
			if (SystemManager.confirmYesNo("Warning","All changes will be lost.\nProceed anyway?") == JOptionPane.YES_OPTION) {
				imageCanvas2D.revertToOriginal();
			}
		} else if (ae.getSource() == addConst) {/*
			 String tmp = JOptionPane.showInputDialog(ICC3D.getDialogParent(), "RGB like this: 255 136 -78");
			 if (tmp!=null) {
			 StringTokenizer token = new StringTokenizer(tmp);
			 int r=0,g=0,b=0, val, i=0;
			 while(token.hasMoreTokens()) {
			 tmp = token.nextToken();
			 val = Integer.parseInt(tmp);
			 switch(i) {
			 case 0: r = val;	break;	
			 case 1: g = val;	break;
			 case 2: b = val;	break;
			 }
			 i++; 
			 }
			 imageCanvas2D.addConst(r,g,b);
			 //TODO canvas.set(image2D.createRenderableImage());
			 }
			 */} else if (ae.getSource() == show3D) {
			this.root.getSessionManager().new3D_View(this);
		} else if (ae.getSource() == imageInfo) {/*
			 JPanel panel = new JPanel();
			 panel.setLayout(new BorderLayout());
			 panel.setBorder(BorderFactory.createTitledBorder(
			 BorderFactory.createEmptyBorder(), "Image attributes"));
			 JTextArea area = new JTextArea();
			 area.setEnabled(false);
			 
			 String imageInfo = new String(
			 "Filename:\t    "+fileName.substring(fileName.lastIndexOf(File.separatorChar)+1, fileName.lastIndexOf("."))
			 + "\nLocation:\t    "+fileName.substring(0, fileName.lastIndexOf(File.separatorChar))
			 + "\nFormat:\t    "+fileName.substring(fileName.lastIndexOf(".")+1, fileName.length()).toUpperCase()
			 + "\nBands:\t    "+imageOriginal.getNumBands()
			 + "\nPixeldepth:\t    "+imageOriginal.getColorModel().getPixelSize()+" bit"
			 );
			 
			 imageInfo += "\nColorspace:\t    ";
			 switch(imageOriginal.getColorModel().getColorSpace().getType()) {
			 case ColorSpace.CS_CIEXYZ:			imageInfo += "CIEXYZ";
			 break;
			 case ColorSpace.CS_GRAY: 				imageInfo += "GRAY";
			 break;
			 case ColorSpace.CS_LINEAR_RGB: 	imageInfo += "LINEAR_RGB";
			 break;
			 case ColorSpace.CS_PYCC:  			imageInfo += "PYCC";
			 break;
			 case ColorSpace.CS_sRGB:  			imageInfo += "sRGB";
			 break;
			 case ColorSpace.TYPE_2CLR:			imageInfo += "Generic 2 component color space";
			 break;
			 case ColorSpace.TYPE_3CLR:  		imageInfo += "Generic 3 component color space";
			 break;
			 case ColorSpace.TYPE_4CLR:  		imageInfo += "Generic 4 component color space";
			 break;
			 case ColorSpace.TYPE_5CLR:  		imageInfo += "Generic 5 component color space";
			 break;
			 case ColorSpace.TYPE_6CLR:  		imageInfo += "Generic 6 component color space";
			 break;
			 case ColorSpace.TYPE_7CLR:  		imageInfo += "Generic 7 component color space";
			 break;
			 case ColorSpace.TYPE_8CLR:  		imageInfo += "Generic 8 component color space";
			 break;
			 case ColorSpace.TYPE_9CLR:  		imageInfo += "Generic 9 component color space";
			 break;
			 case ColorSpace.TYPE_ACLR: 		 	imageInfo += "Generic 10 component color space";
			 break;
			 case ColorSpace.TYPE_BCLR: 		 	imageInfo += "Generic 11 component color space";
			 break;
			 case ColorSpace.TYPE_CCLR: 		 	imageInfo += "Generic 12 component color space";
			 break;
			 case ColorSpace.TYPE_CMY: 			imageInfo += "CMY";
			 break;
			 case ColorSpace.TYPE_CMYK: 			imageInfo += "CMYK";
			 break;
			 case ColorSpace.TYPE_DCLR: 			imageInfo += "Generic 13 component color space";
			 break;
			 case ColorSpace.TYPE_ECLR: 			imageInfo += "Generic 14 component color space";
			 break;
			 case ColorSpace.TYPE_FCLR: 			imageInfo += "Generic 15 component color space";
			 break;
			 case ColorSpace.TYPE_GRAY: 			imageInfo += "GRAY";
			 break;
			 case ColorSpace.TYPE_HLS:  			imageInfo += "HLS";
			 break;
			 case ColorSpace.TYPE_HSV: 			imageInfo += "HSV";
			 break;
			 case ColorSpace.TYPE_Lab: 			imageInfo += "Lab";
			 break;
			 case ColorSpace.TYPE_Luv: 			imageInfo += "Luv";
			 break;
			 case ColorSpace.TYPE_RGB: 			imageInfo += "RGB";
			 break;
			 case ColorSpace.TYPE_XYZ: 			imageInfo += "XYZ";
			 break;
			 case ColorSpace.TYPE_YCbCr: 		imageInfo += "YCbCr";
			 break;
			 case ColorSpace.TYPE_Yxy: 			imageInfo += "Yxy";
			 break;
			 default:												imageInfo += ""
			 +imageOriginal.getColorModel().getColorSpace().getType();
			 break;
			 };

			 imageInfo +=	"\nSize:"+
			 "\n   Width:\t    "+imageOriginal.getWidth()+
			 "\n   Height:\t    "+imageOriginal.getHeight()+
			 "\n   Total pixels:\t    "+
			 (imageOriginal.getWidth()*imageOriginal.getHeight())+
			 "\n\nAlpha supported:  "+
			 ((imageOriginal.getColorModel().hasAlpha()) ? "Yes" : "No")+
			 "\nTransparency:\t    ";
			 
			 switch(imageOriginal.getColorModel().getTransparency()) {
			 case Transparency.BITMASK: 			imageInfo +=
			 "Bitmask\n\t    (either completely opaque, with an " 
			 + "alpha value of 1.0, or completely transparent, with"
			 + " an alpha value of 0.0)";
			 break;
			 case Transparency.OPAQUE:				imageInfo +=
			 "Opaque\n\t    (all pixels have an"
			 +" alpha value of 1.0)";
			 break;
			 case Transparency.TRANSLUCENT:	imageInfo += "Transculent\n\t"
			 +"    (arbitrary alpha values between and "
			 +"including 0.0 and 1.0)";
			 break;
			 };
			 
			 area.setText(imageInfo);
			 panel.add(new JScrollPane(area));
			 
			 JOptionPane.showMessageDialog(ICC3D.getDialogParent(), panel, "Image info, "+title,
			 JOptionPane.INFORMATION_MESSAGE,
			 IconManager.getIcon("binary_question"));
			 */} else if (ae.getSource() == closeSession) {
			// killSession();
			this.dispose();
		}
	}
	
	/**
	 * Returns this session's ImageCanvas2D object.
	 *	
	 * @return ImageCanvas2D
	 * @see ImageCanvas2D
	 */
	public ImageCanvas2D getImageCanvas2D() {
		return imageCanvas2D;	
	}

	/**
	 * Returns this sessions unique identifier.
	 *	
	 * @return session identifier
	 */
	public int getSessionId() {
		return sessionId;	
	}
	
	/**
	 * Returns this sessions unique identifier.
	 *	
	 * @return session identifier
	 */
	public String toString() {
		return Integer.toString(sessionId);	
	}

	/**
	 * Updates the image display with this session's ImageCanvas2D. Any changes
	 * are now visible.
	 */
	public void update() {
		imageCanvas2D.imageUpdate();
	}
	
	/**
	 * Paints the canvas.
	 * @param g The Graphics object associated with the component.
	 */
	public void paint(Graphics g) {
		try {
			super.paint(g);
		} catch (Throwable t) {
			if (t instanceof OutOfMemoryError) {
				SystemManager.displayError(
					"Error",
					"Out of memory. Please restart the application.\nAdjust configuration parameters if necessary");
			}
		}
	}
	
	/**
	 * Signals that the image is selected as the source of a 3D image module.
	 * @param mod The image module.
	 */
	public void addImageModule(ImageModule mod) {
		imageModules.add(mod);
	}
	
	/**
	 * Signals that the image is no longer selected as the source of a 3D image module.
	 * @param mod The image module.
	 */
	public void removeImageModule(ImageModule mod) {
		imageModules.remove(mod);
	}
	
	/**
	 * Returns a icon representation of the image.
	 * @return a icon representation of the image.
	 */
	public ImageIcon getIcon() {
		ImageIcon icon = new ImageIcon(imageCanvas2D.getImage().getScaledInstance(35,
				30, Image.SCALE_SMOOTH));

		return icon;
	}
}
