/*
 ICC3D - Interactive Color Correction in 3 Dimensions.
 Copyright (C) 2002-2003 Gjvik University College.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package icc3dpackage.image2d;


import javax.swing.JPanel;
import javax.media.jai.*;
import java.awt.*;
import java.awt.image.*;
import javax.swing.BorderFactory;


/**
 * This class is used to display thumbnail views of images.
 */
public class ImageCanvas2D_Thumbnail extends JPanel {
	
	/** The parent image canvas */
	private ImageCanvas2D imageCanvas2D;
	
	/** The background color for transparent images */
	private Color backgroundColor;
	
	/** The thumbnail width */
	private int width;
	
	/** The thumbnail height */
	private int height;
	
	/**
	 * Creates a thumbnail of the specified image.
	 * @param fullSize The original image.
	 * @param w The thumbnail width.
	 * @param h The thumbnail height.
	 */
	public ImageCanvas2D_Thumbnail(ImageCanvas2D fullSize, int w, int h) {
		int realW = fullSize.getImageWidth();
		int realH = fullSize.getImageHeight();

		imageCanvas2D = fullSize;
		setBorder(BorderFactory.createEmptyBorder());
		
		if ((float) realW / (float) realH > (float) w / (float) h) {
			width = w;
			height = (int) (realH * (float) width / (float) realW);
		} else {
			height = h;
			width = (int) (realW * (float) height / (float) realH);
		}
		setSize(width, height);
		this.setPreferredSize(new Dimension(width, height));
		this.setMinimumSize(new Dimension(width, height));
		backgroundColor = new Color(0.0f, 0.0f, 0.0f);
	}
	
	/**
	 * Paints the thumbnail image.
	 */
	public void paint(Graphics g) {
		super.paint(g);
		Image imgFull = imageCanvas2D.getImage();

		if (imgFull != null) {
			g.drawImage(imgFull, 0, 0, width, height, 0, 0, imageCanvas2D.width,
					imageCanvas2D.height, backgroundColor, null);
		}
	}
}
