/*
 ICC3D - Interactive Color Correction in 3 Dimensions.
 Copyright (C) 2002-2003 Gjvik University College.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package icc3dpackage.managers;


import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.lang.*;
import java.io.*;


/**
 * This class is used to load and manage images (png-files) used as icons by ICC3D.
 *
 */
public class IconManager extends Manager {
	
	/**
	 * The available icons dynamically loaded by this manager.
	 */
	private static Hashtable icons = null;
	
	/**
	 * Loads icons from default directory: icons.
	 * 
	 * @param output statustext is displayed using this reference.
	 */
	static public void load(JTextArea output) {
		icons = new Hashtable(17);
		int n = 0;

		try {
			File file = new File("icons");
			String classes[] = file.list();

			append(output, "\nLoading icons: ");
			for (int i = 0; i < classes.length; i++) {
				if (classes[i].endsWith(".png") || classes[i].endsWith(".gif")) {
					try {
						String iconName = classes[i].substring(0, classes[i].lastIndexOf("."));

						append(output, "+");
						ImageIcon imageIcon = new ImageIcon("icons" + File.separator + classes[i]);

						icons.put(iconName.toLowerCase(), imageIcon);
						n++;
					} catch (Exception e) {
						append(output, "!");
					}
				}
			}
			append(output, "(" + Integer.toString(n) + ")");
		} catch (Exception e) {}					
	}
	
	/**
	 * Retrieves a requested ImageIcon.
	 *
	 * <p>For example: 
	 *	Name of icon from storage media: "icons/myIcon.png"
	 * The following code retrieves myIcon.png as an ImageIcon:
	 *	IconManager.getIcon("myicon");
	 *
	 * @param key name of requeste ImageIcon in lower-case NB: lower-case
	 * @return Requested ImageIcon if found, otherwise null
	 * @see javax.swing.ImageIcon
	 */
	static public ImageIcon getIcon(String key) {
		if (icons==null) {
			load(new JTextArea());
		}
		return (ImageIcon) icons.get(key);
	}
	
	/**
	 * Adds a new ImageIcone to this IconManager. Neither the key nor the icon can be null.
	 *
	 * @param key name to assign to this new icon
	 *	@param icon ImageIcon to be added
	 *	@return the previous ImageIcon of the specified key, or null if it did not have one.
	 */
	static public ImageIcon add(String key, ImageIcon icon) {
		ImageIcon tmp = null;

		try {
			tmp = (ImageIcon) icons.put(key, icon);
		} catch (NullPointerException npe) {
			tmp = null;
		}
		return tmp;
	}
	
	/**
	 * Removes an ImageIcon from this IconManager.
	 *
	 * @param key name of icon to remove
	 *	@return requested ImageIcon if found, otherwise null
	 */	
	static public ImageIcon remove(String key) {
		return (ImageIcon) icons.remove(key);
	}
}
