/*
 ICC3D - Interactive Color Correction in 3 Dimensions.
 Copyright (C) 2002-2003 Gjvik University College.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package icc3dpackage.managers;


import java.util.Vector;
import java.awt.BorderLayout;
import javax.swing.*;

import icc3dpackage.*;
import icc3dpackage.image2d.*;
import icc3dpackage.modules.*;
import icc3dpackage.quantization.*;
import icc3dpackage.view3d.*;


/**
 * The SessionManager keeps track of all open images and 3D views.
 */
public class SessionManager {	
	
	/** The images */
	private Vector images;
	
	/** The 3D views */
	private Vector view3Ds;
	
	/** The last image */
	private int lastImage;
	
	/** The last 3D view */
	private int last3D_View;
	
	/** The main application */
	private ICC3D icc3d;
	
	/**
	 * Creates a SessionManager for the given application.
	 * @param icc3d The main application.
	 */
	public SessionManager(ICC3D icc3d) {
		this.icc3d = icc3d;
		images = new Vector();
		view3Ds = new Vector();
	}
	
	/**
	 * Checks if the application is ready to close (Currently not used).
	 * @return True if the application can close.
	 */
	public boolean quit() {
		return true;
	}
	
	/**
	 * Creates a new 2D image.
	 * @return The Image2D if created.
	 */
	public Image2D newImage2D() {
		return newImage2DQuantized();
	}
	
	/**
	 * Creates a new quantized 2D image.
	 * @return The Image2D if created.
	 */
	public Image2D newImage2DQuantized() {
		ImageOpenParameter p = ImageOpenDialog.open();

		if (p.imageReadSuccessfully()) {
			lastImage++;
			
			if (p.getQuantize()) {
				Quantization q = new Quantization(Quantization.IMAGE_QUANTIZATION);
				JCheckBox colorMatch = new JCheckBox("Find closest match");

				colorMatch.setSelected(true);
				JCheckBox errorDiffusion = new JCheckBox("Apply error diffusion method 1");

				errorDiffusion.setSelected(true);
				JCheckBox errorDiffusion2 = new JCheckBox("Apply error diffusion method 2");
				JPanel panel = new JPanel();

				panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
				panel.add(q.getParameterPanel());
				panel.add(colorMatch);
				panel.add(errorDiffusion);
				panel.add(errorDiffusion2);
				JOptionPane.showMessageDialog(ICC3D.getDialogParent(), panel);
				QuantizedObject qo = new QuantizedObject();
				int originalPoints = p.getInput().length / 3;

				qo.numPoints = originalPoints;
				qo.points = p.getInput(); // /*ColorSpaceManager.getColorSpace("Lab").fromCIEXYZ(*/p.getXYZ()/*)*/;
				qo.pointToPixelCount = new int[p.getColorToPixelIndex().length];
				for (int i = 0; i < qo.pointToPixelCount.length; i++) {
					qo.pointToPixelCount[i] = p.getColorToPixelIndex()[i].length;
				}
				qo = q.quantize(qo);
				// qo.points = p.getInputColorSpace3D().fromCIEXYZ(/*ColorSpaceManager.getColorSpace("Lab").toCIEXYZ(*/qo.points/*)*/);
				if (qo.pointToColorCount != null) {
					ImageOpenParameter newP = new ImageOpenParameter();
					int pixelToColorIndex[] = new int[p.getWidth() * p.getHeight()];
					int colorToPixelIndex[][] = new int[qo.numPoints][];
					int inArray = 0;
					int curColor = 0;
					int cToP[];

					for (int i = 0; i < qo.numPoints; i++) {
						inArray = 0;
						colorToPixelIndex[i] = new int[qo.pointToPixelCount[i]];
						for (int j = 0; j < qo.pointToColorCount[i]; j++) {
							curColor = qo.pointToColorIndex[i][j];
							cToP = p.getColorToPixelIndex()[curColor];
							for (int k = 0; k < cToP.length; k++) {
								colorToPixelIndex[i][inArray++] = cToP[k];
								pixelToColorIndex[cToP[k]] = i;
							}
						}
					}
					
					int numNewPoints = qo.numPoints;
					int oldPixelToColorIndex[] = p.getPixelToColorIndex();
					float newPointData[] = qo.points;
					float oldPointData[] = p.getInput();
					int colorToPixelCount[] = qo.pointToPixelCount;
					int oldColorToPixelIndex[][] = p.getColorToPixelIndex();
					boolean exactMatch = colorMatch.isSelected();
					boolean diffuse1 = errorDiffusion.isSelected();

					if (exactMatch || diffuse1) {
						int nearestPoint, nearPoint;
						float lowestRadius, lowRadius;
						float r1, r2;
						int qq, pp;

						for (int i = 0; i < originalPoints; i++) {
							qq = i * 3;
							r1 = (newPointData[0] - oldPointData[qq])
											* (newPointData[0] - oldPointData[qq])
									+ (newPointData[1] - oldPointData[qq + 1])
											* (newPointData[1] - oldPointData[qq + 1])
									+ (newPointData[2] - oldPointData[qq + 2])
											* (newPointData[2] - oldPointData[qq + 2]);
							r2 = (newPointData[3] - oldPointData[qq])
											* (newPointData[3] - oldPointData[qq])
									+ (newPointData[4] - oldPointData[qq + 1])
											* (newPointData[4] - oldPointData[qq + 1])
									+ (newPointData[5] - oldPointData[qq + 2])
											* (newPointData[5] - oldPointData[qq + 2]);
							   
							if (r1 < r2) {
								lowestRadius = r1;
								lowRadius = r2;
								nearestPoint = 0;
								nearPoint = 1;
							} else {
								lowestRadius = r2;
								lowRadius = r1;
								nearestPoint = 1;
								nearPoint = 0;
							}
							for (int j = 2; j < numNewPoints; j++) {
								pp = j * 3;
								r1 = (newPointData[pp] - oldPointData[qq])
												* (newPointData[pp] - oldPointData[qq])
										+ (newPointData[pp + 1] - oldPointData[qq + 1])
												* (newPointData[pp + 1] - oldPointData[qq + 1])
										+ (newPointData[pp + 2] - oldPointData[qq + 2])
												* (newPointData[pp + 2] - oldPointData[qq + 2]);
							  
								if (r1 < lowestRadius) {
									lowRadius = lowestRadius;
									nearPoint = nearestPoint;
									nearestPoint = j;
									lowestRadius = r1;
								} else if (r1 < lowRadius) {
									lowRadius = r1;
									nearPoint = j;
								}
							}
							lowestRadius = (float) Math.sqrt(lowestRadius);
							lowRadius = (float) Math.sqrt(lowRadius);
							float higprob = lowRadius / (lowRadius + lowestRadius);
							// higprob = (float)Math.pow(higprob, 1.3);
							int colorPixels[] = oldColorToPixelIndex[i];
							int pixel;

							for (int j = 0; j < colorPixels.length; j++) {
								pixel = colorPixels[j];

								/* if (pixel%p.getWidth()==0) {
								 System.out.println("Old color "+oldPointData[qq]+", "+oldPointData[qq+1]+", "+oldPointData[qq+2]);
								 System.out.println("\nrLowest "+lowestRadius+", rLow "+lowRadius);
								 System.out.println("Prob: "+higprob);
								 System.out.println("nearestPoint "+nearestPoint+": "+newPointData[nearestPoint*3]+","+newPointData[nearestPoint*3+1]+","+newPointData[nearestPoint*3+2]);
								 System.out.println("nearPoint "+nearPoint+": "+newPointData[nearPoint*3]+","+newPointData[nearPoint*3+1]+","+newPointData[nearPoint*3+2]);
								 }*/
								if ((float) Math.random() < higprob || !diffuse1) {
									pixelToColorIndex[pixel] = nearestPoint;
								} else {
									pixelToColorIndex[pixel] = nearPoint;
								}
							}
						}	
					}
					
					boolean obfuscate = true;

					if (obfuscate) {
						float xError[] = new float[pixelToColorIndex.length];
						float yError[] = new float[pixelToColorIndex.length];
						float zError[] = new float[pixelToColorIndex.length];
						float errorX, errorY, errorZ;
						float originalX, originalY, originalZ;
						float errorWeight[] = ErrorDiffusion.FLOYD_STEINBERG_WEIGHTS;
						int errorIndices[][] = ErrorDiffusion.FLOYD_STEINBERG_INDICES;
						float wantedX, wantedY, wantedZ;
						// float newX, newY, newZ;
						int pp;
						int width = p.getWidth();
						int height = p.getHeight();
						float r1;
						
						for (int kl = 0; kl < 2; kl++) {
							for (int j = 0; j < height; j++) {
								for (int i = 0; i < width; i++) {
									// System.out.println("errors "+xError[j*width+i]+","+yError[j*width+i]+","+zError[j*width+i]);
									wantedX = xError[j * width + i]
											+ oldPointData[oldPixelToColorIndex[j * width + i] * 3];
									wantedY = yError[j * width + i]
											+ oldPointData[oldPixelToColorIndex[j * width + i] * 3 + 1];
									wantedZ = zError[j * width + i]
											+ oldPointData[oldPixelToColorIndex[j * width + i] * 3 + 2];
								
									int nearestPoint = 0;
									float lowestRadius = (newPointData[0] - wantedX)
													* (newPointData[0] - wantedX)
											+ (newPointData[1] - wantedY) * (newPointData[1] - wantedY)
											+ (newPointData[2] - wantedZ) * (newPointData[2] - wantedZ);

									for (int k = 1; k < numNewPoints; k++) {
										pp = k * 3;
										r1 = (newPointData[pp] - wantedX) * (newPointData[pp] - wantedX)
												+ (newPointData[pp + 1] - wantedY)
														* (newPointData[pp + 1] - wantedY)
												+ (newPointData[pp + 2] - wantedZ)
														* (newPointData[pp + 2] - wantedZ);
								  
										if (r1 < lowestRadius) {
											nearestPoint = k;
											lowestRadius = r1;
										}
									}

									/* if (pixelToColorIndex[j*width+i]!=nearestPoint) {
									 System.out.println("changed pixel "+i+","+j);
									 }*/
									pixelToColorIndex[j * width + i] = nearestPoint;
									errorX = wantedX - newPointData[nearestPoint * 3];
									errorY = wantedY - newPointData[nearestPoint * 3 + 1];
									errorZ = wantedZ - newPointData[nearestPoint * 3 + 2];
									// errorX *= 100.0f;
									// errorY *= 100.0f;
									// errorZ *= 100.0f;
									for (int k = 0; k < errorIndices.length; k++) {
										int errorPosX = i + errorIndices[k][0];
										int errorPosY = j + errorIndices[k][1];

										if (errorPosX >= 0 && errorPosX < width && errorPosY >= 0
												&& errorPosY < height) {
											xError[errorPosY * width + errorPosX] = errorX * errorWeight[k];
											yError[errorPosY * width + errorPosX] = errorY * errorWeight[k];
											zError[errorPosY * width + errorPosX] = errorZ * errorWeight[k];
										}
									}
								}
							}
						}
					}
					
					// boolean useSecondMethod = false;
					if (errorDiffusion2.isSelected()) {
						// for (int kl=0;kl<2;kl++) {
						int pp;
						float r1, r2;
						int nearestPoint, nearPoint;
						float lowestRadius, lowRadius;
						
						int widthMinusOne = p.getWidth() - 1;
						int width = p.getWidth();
						int heightMinusOne = p.getHeight() - 1;
						
						float sumOriginalX, sumOriginalY, sumOriginalZ;
						float sumNewX, sumNewY, sumNewZ;
						float wantedX, wantedY, wantedZ;
						float twoOver36 = 2.0f / 36.0f;
						float threeOver36 = 3.0f / 36.0f;
						float centerWeight = 1.0f - twoOver36 * 4.0f - threeOver36 * 4.0f;

						for (int j = 1; j < heightMinusOne; j++) {
							for (int i = 1; i < widthMinusOne; i++) {
								sumOriginalX = twoOver36
												* oldPointData[oldPixelToColorIndex[(j - 1) * width + i - 1] * 3]
										+ twoOver36
												* oldPointData[oldPixelToColorIndex[(j - 1) * width + i + 1] * 3]
										+ twoOver36
												* oldPointData[oldPixelToColorIndex[(j + 1) * width + i + 1] * 3]
										+ twoOver36
												* oldPointData[oldPixelToColorIndex[(j + 1) * width + i - 1] * 3]
										+ threeOver36
												* oldPointData[oldPixelToColorIndex[(j - 1) * width + i] * 3]
										+ threeOver36
												* oldPointData[oldPixelToColorIndex[(j) * width + i - 1] * 3]
										+ threeOver36
												* oldPointData[oldPixelToColorIndex[(j) * width + i + 1] * 3]
										+ threeOver36
												* oldPointData[oldPixelToColorIndex[(j + 1) * width + i] * 3]
										+ centerWeight
												* oldPointData[oldPixelToColorIndex[(j) * width + i] * 3];
								sumOriginalY = twoOver36
												* oldPointData[oldPixelToColorIndex[(j - 1) * width + i - 1] * 3 + 1]
										+ twoOver36
												* oldPointData[oldPixelToColorIndex[(j - 1) * width + i + 1] * 3 + 1]
										+ twoOver36
												* oldPointData[oldPixelToColorIndex[(j + 1) * width + i + 1] * 3 + 1]
										+ twoOver36
												* oldPointData[oldPixelToColorIndex[(j + 1) * width + i - 1] * 3 + 1]
										+ threeOver36
												* oldPointData[oldPixelToColorIndex[(j - 1) * width + i] * 3 + 1]
										+ threeOver36
												* oldPointData[oldPixelToColorIndex[(j) * width + i - 1] * 3 + 1]
										+ threeOver36
												* oldPointData[oldPixelToColorIndex[(j) * width + i + 1] * 3 + 1]
										+ threeOver36
												* oldPointData[oldPixelToColorIndex[(j + 1) * width + i] * 3 + 1]
										+ centerWeight
												* oldPointData[oldPixelToColorIndex[(j) * width + i] * 3 + 1];
								sumOriginalZ = twoOver36
												* oldPointData[oldPixelToColorIndex[(j - 1) * width + i - 1] * 3 + 2]
										+ twoOver36
												* oldPointData[oldPixelToColorIndex[(j - 1) * width + i + 1] * 3 + 2]
										+ twoOver36
												* oldPointData[oldPixelToColorIndex[(j + 1) * width + i + 1] * 3 + 2]
										+ twoOver36
												* oldPointData[oldPixelToColorIndex[(j + 1) * width + i - 1] * 3 + 2]
										+ threeOver36
												* oldPointData[oldPixelToColorIndex[(j - 1) * width + i] * 3 + 2]
										+ threeOver36
												* oldPointData[oldPixelToColorIndex[(j) * width + i - 1] * 3 + 2]
										+ threeOver36
												* oldPointData[oldPixelToColorIndex[(j) * width + i + 1] * 3 + 2]
										+ threeOver36
												* oldPointData[oldPixelToColorIndex[(j + 1) * width + i] * 3 + 2]
										+ centerWeight
												* oldPointData[oldPixelToColorIndex[(j) * width + i] * 3 + 2];
								sumNewX = twoOver36
												* newPointData[pixelToColorIndex[(j - 1) * width + i - 1] * 3]
										+ twoOver36
												* newPointData[pixelToColorIndex[(j - 1) * width + i + 1] * 3]
										+ twoOver36
												* newPointData[pixelToColorIndex[(j + 1) * width + i + 1] * 3]
										+ twoOver36
												* newPointData[pixelToColorIndex[(j + 1) * width + i - 1] * 3]
										+ threeOver36
												* newPointData[pixelToColorIndex[(j - 1) * width + i] * 3]
										+ threeOver36
												* newPointData[pixelToColorIndex[(j) * width + i - 1] * 3]
										+ threeOver36
												* newPointData[pixelToColorIndex[(j) * width + i + 1] * 3]
										+ threeOver36
												* newPointData[pixelToColorIndex[(j + 1) * width + i] * 3];
								sumNewY = twoOver36
												* newPointData[pixelToColorIndex[(j - 1) * width + i - 1] * 3 + 1]
										+ twoOver36
												* newPointData[pixelToColorIndex[(j - 1) * width + i + 1] * 3 + 1]
										+ twoOver36
												* newPointData[pixelToColorIndex[(j + 1) * width + i + 1] * 3 + 1]
										+ twoOver36
												* newPointData[pixelToColorIndex[(j + 1) * width + i - 1] * 3 + 1]
										+ threeOver36
												* newPointData[pixelToColorIndex[(j - 1) * width + i] * 3 + 1]
										+ threeOver36
												* newPointData[pixelToColorIndex[(j) * width + i - 1] * 3 + 1]
										+ threeOver36
												* newPointData[pixelToColorIndex[(j) * width + i + 1] * 3 + 1]
										+ threeOver36
												* newPointData[pixelToColorIndex[(j + 1) * width + i] * 3 + 1];
								sumNewZ = twoOver36
												* newPointData[pixelToColorIndex[(j - 1) * width + i - 1] * 3 + 2]
										+ twoOver36
												* newPointData[pixelToColorIndex[(j - 1) * width + i + 1] * 3 + 2]
										+ twoOver36
												* newPointData[pixelToColorIndex[(j + 1) * width + i + 1] * 3 + 2]
										+ twoOver36
												* newPointData[pixelToColorIndex[(j + 1) * width + i - 1] * 3 + 2]
										+ threeOver36
												* newPointData[pixelToColorIndex[(j - 1) * width + i] * 3 + 2]
										+ threeOver36
												* newPointData[pixelToColorIndex[(j) * width + i - 1] * 3 + 2]
										+ threeOver36
												* newPointData[pixelToColorIndex[(j) * width + i + 1] * 3 + 2]
										+ threeOver36
												* newPointData[pixelToColorIndex[(j + 1) * width + i] * 3 + 2];
								wantedX = (sumOriginalX - sumNewX) / centerWeight;
								wantedY = (sumOriginalY - sumNewY) / centerWeight;
								wantedZ = (sumOriginalZ - sumNewZ) / centerWeight;
								
								// qq = i*3;
								r1 = (newPointData[0] - wantedX) * (newPointData[0] - wantedX)
										+ (newPointData[1] - wantedY) * (newPointData[1] - wantedY)
										+ (newPointData[2] - wantedZ) * (newPointData[2] - wantedZ);
								r2 = (newPointData[3] - wantedX) * (newPointData[3] - wantedX)
										+ (newPointData[4] - wantedY) * (newPointData[4] - wantedY)
										+ (newPointData[5] - wantedZ) * (newPointData[5] - wantedZ);
								
								if (r1 < r2) {
									lowestRadius = r1;
									lowRadius = r2;
									nearestPoint = 0;
									nearPoint = 1;
								} else {
									lowestRadius = r2;
									lowRadius = r1;
									nearestPoint = 1;
									nearPoint = 0;
								}
								for (int k = 2; k < numNewPoints; k++) {
									pp = k * 3;
									r1 = (newPointData[pp] - wantedX) * (newPointData[pp] - wantedX)
											+ (newPointData[pp + 1] - wantedY) * (newPointData[pp + 1] - wantedY)
											+ (newPointData[pp + 2] - wantedZ) * (newPointData[pp + 2] - wantedZ);
								  
									if (r1 < lowestRadius) {
										lowRadius = lowestRadius;
										nearPoint = nearestPoint;
										nearestPoint = k;
										lowestRadius = r1;
									} else if (r1 < lowRadius) {
										lowRadius = r1;
										nearPoint = k;
									}
								}
								lowestRadius = (float) Math.sqrt(lowestRadius);
								lowRadius = (float) Math.sqrt(lowRadius);
								float higprob = lowRadius / (lowRadius + lowestRadius);

								// higprob = (float)Math.pow(higprob, 1.5);
								if (Math.random() < higprob) {
									pixelToColorIndex[(j) * width + i] = nearestPoint;
								} else {
									pixelToColorIndex[(j) * width + i] = nearPoint;
								}
							}
						}
						// }
					}
					
					// boolean useSecondMethod = false;
					boolean obfuscate2 = false;

					if (obfuscate2) {
						// for (int kl=0;kl<2;kl++) {
						int pp;
						float r1, r2;
						int nearestPoint, nearPoint;
						float lowestRadius, lowRadius;
						
						int widthMinusOne = p.getWidth() - 1;
						int width = p.getWidth();
						int heightMinusOne = p.getHeight() - 1;
						
						// float sumOriginalX, sumOriginalY, sumOriginalZ;
						float totalErrorX, totalErrorY, totalErrorZ;
						float cornerWeight = 0.4f;
						float neighborWeight = 0.4f;
						
						float sumNewX, sumNewY, sumNewZ;
						float wantedX, wantedY, wantedZ;

						// float twoOver36 = 2.0f/36.0f;
						// float threeOver36 = 3.0f/36.0f;
						// float centerWeight = 1.0f - twoOver36*4.0f - threeOver36*4.0f;
						for (int j = 1; j < heightMinusOne; j++) {
							for (int i = 1; i < widthMinusOne; i++) {
								totalErrorX = cornerWeight * (oldPointData[oldPixelToColorIndex[(j - 1) * width + i - 1] * 3] - newPointData[pixelToColorIndex[(j - 1) * width + i - 1] * 3] + oldPointData[oldPixelToColorIndex[(j - 1) * width + i + 1] * 3] - newPointData[pixelToColorIndex[(j - 1) * width + i + 1] * 3] + oldPointData[oldPixelToColorIndex[(j + 1) * width + i - 1] * 3] - newPointData[pixelToColorIndex[(j + 1) * width + i - 1] * 3] + oldPointData[oldPixelToColorIndex[(j + 1) * width + i + 1] * 3] - newPointData[pixelToColorIndex[(j + 1) * width + i + 1] * 3]
										) + neighborWeight * (oldPointData[oldPixelToColorIndex[(j - 1) * width + i] * 3] - newPointData[pixelToColorIndex[(j - 1) * width + i] * 3] + oldPointData[oldPixelToColorIndex[(j) * width + i - 1] * 3] - newPointData[pixelToColorIndex[(j) * width + i - 1] * 3] + oldPointData[oldPixelToColorIndex[(j) * width + i + 1] * 3] - newPointData[pixelToColorIndex[(j) * width + i + 1] * 3] + oldPointData[oldPixelToColorIndex[(j + 1) * width + i] * 3] - newPointData[pixelToColorIndex[(j + 1) * width + i] * 3]
										);
								totalErrorY = cornerWeight * (oldPointData[oldPixelToColorIndex[(j - 1) * width + i - 1] * 3 + 1] - newPointData[pixelToColorIndex[(j - 1) * width + i - 1] * 3 + 1] + oldPointData[oldPixelToColorIndex[(j - 1) * width + i + 1] * 3 + 1] - newPointData[pixelToColorIndex[(j - 1) * width + i + 1] * 3 + 1] + oldPointData[oldPixelToColorIndex[(j + 1) * width + i - 1] * 3 + 1] - newPointData[pixelToColorIndex[(j + 1) * width + i - 1] * 3 + 1] + oldPointData[oldPixelToColorIndex[(j + 1) * width + i + 1] * 3 + 1] - newPointData[pixelToColorIndex[(j + 1) * width + i + 1] * 3 + 1]
										) + neighborWeight * (oldPointData[oldPixelToColorIndex[(j - 1) * width + i] * 3 + 1] - newPointData[pixelToColorIndex[(j - 1) * width + i] * 3 + 1] + oldPointData[oldPixelToColorIndex[(j) * width + i - 1] * 3 + 1] - newPointData[pixelToColorIndex[(j) * width + i - 1] * 3 + 1] + oldPointData[oldPixelToColorIndex[(j) * width + i + 1] * 3 + 1] - newPointData[pixelToColorIndex[(j) * width + i + 1] * 3 + 1] + oldPointData[oldPixelToColorIndex[(j + 1) * width + i] * 3 + 1] - newPointData[pixelToColorIndex[(j + 1) * width + i] * 3 + 1]
										);
																						
								totalErrorZ = cornerWeight * (oldPointData[oldPixelToColorIndex[(j - 1) * width + i - 1] * 3 + 2] - newPointData[pixelToColorIndex[(j - 1) * width + i - 1] * 3 + 2] + oldPointData[oldPixelToColorIndex[(j - 1) * width + i + 1] * 3 + 2] - newPointData[pixelToColorIndex[(j - 1) * width + i + 1] * 3 + 2] + oldPointData[oldPixelToColorIndex[(j + 1) * width + i - 1] * 3 + 2] - newPointData[pixelToColorIndex[(j + 1) * width + i - 1] * 3 + 2] + oldPointData[oldPixelToColorIndex[(j + 1) * width + i + 1] * 3 + 2] - newPointData[pixelToColorIndex[(j + 1) * width + i + 1] * 3 + 2]
										) + neighborWeight * (oldPointData[oldPixelToColorIndex[(j - 1) * width + i] * 3 + 2] - newPointData[pixelToColorIndex[(j - 1) * width + i] * 3 + 2] + oldPointData[oldPixelToColorIndex[(j) * width + i - 1] * 3 + 2] - newPointData[pixelToColorIndex[(j) * width + i - 1] * 3 + 2] + oldPointData[oldPixelToColorIndex[(j) * width + i + 1] * 3 + 2] - newPointData[pixelToColorIndex[(j) * width + i + 1] * 3 + 2] + oldPointData[oldPixelToColorIndex[(j + 1) * width + i] * 3 + 2] - newPointData[pixelToColorIndex[(j + 1) * width + i] * 3 + 2]
										);
								             
								wantedX = oldPointData[oldPixelToColorIndex[(j) * width + i] * 3]
										+ totalErrorX;
								wantedY = oldPointData[oldPixelToColorIndex[(j) * width + i] * 3 + 1]
										+ totalErrorY;
								wantedZ = oldPointData[oldPixelToColorIndex[(j) * width + i] * 3 + 2]
										+ totalErrorZ;
								// qq = i*3;
								r1 = (newPointData[0] - wantedX) * (newPointData[0] - wantedX)
										+ (newPointData[1] - wantedY) * (newPointData[1] - wantedY)
										+ (newPointData[2] - wantedZ) * (newPointData[2] - wantedZ);
								r2 = (newPointData[3] - wantedX) * (newPointData[3] - wantedX)
										+ (newPointData[4] - wantedY) * (newPointData[4] - wantedY)
										+ (newPointData[5] - wantedZ) * (newPointData[5] - wantedZ);
								
								if (r1 < r2) {
									lowestRadius = r1;
									lowRadius = r2;
									nearestPoint = 0;
									nearPoint = 1;
								} else {
									lowestRadius = r2;
									lowRadius = r1;
									nearestPoint = 1;
									nearPoint = 0;
								}
								for (int k = 2; k < numNewPoints; k++) {
									pp = k * 3;
									r1 = (newPointData[pp] - wantedX) * (newPointData[pp] - wantedX)
											+ (newPointData[pp + 1] - wantedY) * (newPointData[pp + 1] - wantedY)
											+ (newPointData[pp + 2] - wantedZ) * (newPointData[pp + 2] - wantedZ);
								  
									if (r1 < lowestRadius) {
										lowRadius = lowestRadius;
										nearPoint = nearestPoint;
										nearestPoint = k;
										lowestRadius = r1;
									} else if (r1 < lowRadius) {
										lowRadius = r1;
										nearPoint = k;
									}
								}
								lowestRadius = (float) Math.sqrt(lowestRadius);
								lowRadius = (float) Math.sqrt(lowRadius);
								float higprob = lowRadius / (lowRadius + lowestRadius);

								// higprob = (float)Math.pow(higprob, 1.5);
								if (Math.random() < higprob) {
									pixelToColorIndex[(j) * width + i] = nearestPoint;
								} else {
									pixelToColorIndex[(j) * width + i] = nearPoint;
								}
							}
						}
						// }
					}
					
					if (errorDiffusion.isSelected() || errorDiffusion2.isSelected()
							|| obfuscate || obfuscate2 || exactMatch) {
						int added[] = new int[colorToPixelCount.length];
						int sum[] = new int[colorToPixelCount.length];

						for (int i = 0; i < pixelToColorIndex.length; i++) {
							sum[pixelToColorIndex[i]]++;
						}
						
						for (int i = 0; i < colorToPixelCount.length; i++) {
							// System.out.println("Color "+i+" count: "+colorToPixelCount[i]);
							colorToPixelIndex[i] = new int[sum[i]];
						}
						for (int i = 0; i < pixelToColorIndex.length; i++) {
							colorToPixelIndex[pixelToColorIndex[i]][added[pixelToColorIndex[i]]++] = i;
						}
					}
					newP.set(p.getInputColorSpace3D(), p.getWidth(), p.getHeight(),
							p.getInputColorSpace3D().toCIEXYZ(qo.points), qo.points,
							pixelToColorIndex, colorToPixelIndex, true);
					
					Image2D image = new Image2D(icc3d, lastImage, newP);

					images.add(image);
					return image;
				} else {
					Image2D image = new Image2D(icc3d, lastImage, p);

					images.add(image);
					return image;
				}
			} else {
				Image2D image = new Image2D(icc3d, lastImage, p);

				images.add(image);
				return image;
			}
		}
		return null;
	}
	
	/**
	 * Removes the 2D image.
	 * @param image The image.
	 */
	public void removeImage2D(Image2D image) {
		images.remove(image);
	}
	
	/**
	 * Creates a new 3D view.
	 * @return The View3D.
	 */
	public View3D new3D_View() {
		View3D view = new View3D(icc3d, ++last3D_View);
		view3Ds.add(view);
		return view;
	}
	
	/**
	 * Creates a new 3D view of the given image.
	 * @return The View3D.
	 */
	public View3D new3D_View(Image2D image) {
		View3D view = new3D_View();
		ImageModule imModule = new ImageModule();

		view.addModule(imModule);
		imModule.setSource(image);
		view.update();
		return view;
	}
	
	/**
	 * Removes the 3D view.
	 * @param view The 3D view.
	 */
	public void remove3D_View(View3D view) {
		view3Ds.remove(view);
	}
	
	/**
	 * Retrieves a list of all images,
	 * @return An array of images.
	 */
	public Image2D[] getImages() {
		Image2D imgs[] = new Image2D[images.size()];

		for (int i = 0; i < images.size(); i++) {
			imgs[i] = (Image2D) images.elementAt(i);
		}
		return imgs;
	}
	
	/**
	 * Retrieves a list of all 3D views,
	 * @return An array of 3D views.
	 */
	public View3D[] getView3Ds() {
		View3D views[] = new View3D[view3Ds.size()];

		for (int i = 0; i < view3Ds.size(); i++) {
			views[i] = (View3D) view3Ds.elementAt(i);
		}
		return views;
	}
	
	/**
	 * Returns the image with the given title.
	 * @param image The image title.
	 * @return The Image2D.
	 */
	public Image2D getImage(String image) {
		for (int i = 0; i < images.size(); i++) {
			if (image.equals(((Image2D) images.elementAt(i)).getTitle())) {
				return (Image2D) images.elementAt(i);
			}
		}
		return null;
	}
}
