/*
 ICC3D - Interactive Color Correction in 3 Dimensions.
 Copyright (C) 2002-2003 Gjvik University College.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package icc3dpackage.managers;


import icc3dpackage.visualizations.generic.*;
import icc3dpackage.visualizations.*;
import javax.swing.*;
import java.util.*;
import java.io.*;


/**
 * This class is used to load and manage Visualization3D objects.
 *
 */
public abstract class VisualizationManager extends Manager {

	/** All the managed plugins */
	protected static Manageable manageable;
	
	/**
	 * Loads Visualization3D plugins dynamically from default directory: 
	 * icc3dpackage/visualizations.
	 * 
	 * @param output statustext is displayed using this reference.
	 * @see #load
	 */
	static public void load(JTextArea output) {
		manageable = new Manageable();
		Class c = null;

		try {
			c = Class.forName("icc3dpackage.visualizations.generic.Visualization3D");
		} catch (Exception e) {
			SystemManager.displayError(
				"Error",
				"Unable to find the Visualization3D class");
			System.exit(1);
		}
		load(output, c, "icc3dpackage" + File.separator + "visualizations",
				"visualizations", manageable);
	}
	
	/**
	 * Retrieves a requested Visualization3D object from this VisualizationManager.
	 *
	 * @param name of visualization
	 * @return requested Visualization3D, or null if not found
	 *	@see #getNames
	 */
	public static Visualization3D getVisualization(String name) {
		return (Visualization3D) getPlugin(name, manageable);
	}
	
	/**
	 * Returns the names of the available visualizations.
	 * @return The names of the available visualizations.
	 */
	public static String[] getNames() {
		return getNames(manageable);
	}
}
