/*
 ICC3D - Interactive Color Correction in 3 Dimensions.
 Copyright (C) 2002-2003 Gjvik University College.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package icc3dpackage.modules;


import javax.swing.*;
import javax.media.j3d.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.vecmath.*;

import icc3dpackage.*;
import icc3dpackage.colorspaces.generic.ColorSpace3D;
import icc3dpackage.managers.*;
import icc3dpackage.modules.generic.*;
import icc3dpackage.parser.*;
import icc3dpackage.utils.*;
import icc3dpackage.view3d.*;
import icc3dpackage.visualizations.*;
import icc3dpackage.visualizations.generic.Visualization3D;


/**
 * This class is used to compare the gamuts of different kinds of devices
 * and images.
 */
public class CompareGamutModule extends Module {
	
	/** The active visualizations. */
	private Visualization3D visualization3D[];
		
	/** The names of the available visualizations. */
	private Vector visualizationNames[];
	
	/** The available visualizations. */
	private Vector visualizations[];
	
	/** The gamut data as read from a file. */
	private float gamutData[][];
	
	/** A Lab color space. */
	private ColorSpace3D labColorSpace3D;
		
	/** The parameter panel. */
	private JPanel parameterPanel = null;
	
	/** Splits the parameter panel in two parts. */ 
	private JSplitPane jsp;
	
	/** The combination type. */
	private JComboBox modeBox;
	
	/** Used to select a visualization. */
	private JComboBox gamutCombo;
	
	/** Used to select a visualization. */
	private JComboBox gamutCombo1;
	
	/** Used to select a visualization. */
	private JComboBox gamutCombo2;
	
	/** Used to select the quantization detail. */
	private JComboBox quantize;
	
	/** The combined gamut volume. */
	private JLabel gamutVolume;
	
	/** Select gamut A. */
	private JButton gamut1;
	
	/** Select gamut B. */
	private JButton gamut2;
	
	/** The View3D parent. */
	private View3D view3D;

	/** Used to select a gamut data source. */
	private JMenuItem items[];
	
	/** Used to select a gamut data source. */
	private float dataItems[][];
	
	/** Module to gamut item index. */
	private int modToItemIndex[];
	
	/** The tabbed pane containing the visualizations. */
	private JTabbedPane tabs;
	
	/** A visualization split pane. */
	private JSplitPane aSplit;
	
	/** A visualization split pane. */
	private JSplitPane bSplit;
	
	/** A visualization split pane. */
	private JSplitPane cSplit;
	
	/**
	 * Default constructor.
	 */
	public CompareGamutModule() {}
	
	/**
	 * Creates the parameter panel for this GamutView.
	 */
	private void createParameterPanel() {
		aSplit = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
		aSplit.setBorder(BorderFactory.createEmptyBorder());
		aSplit.setEnabled(false);
		bSplit = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
		bSplit.setBorder(BorderFactory.createEmptyBorder());
		bSplit.setEnabled(false);
		cSplit = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
		cSplit.setBorder(BorderFactory.createEmptyBorder());
		cSplit.setEnabled(false);
		tabs = new JTabbedPane();
		tabs.addTab("A", aSplit);
		tabs.addTab("B", bSplit);
		tabs.addTab("Combined", cSplit);
		parameterPanel = new JPanel(new BorderLayout());
		jsp = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
		jsp.setBorder(BorderFactory.createEmptyBorder());
		jsp.setEnabled(false);
		jsp.setBottomComponent(tabs);
		gamut1 = new JButton("(Gamut A)");
		gamut2 = new JButton("(Gamut B)");
		quantize = new JComboBox(new String[] {"16", "32", "48", "64", "96", "128"});
		modeBox = new JComboBox(new String[] {"A", "B", "A or B", "A and B",
			"A minus B", "B minus A", "A xor B"});
		modeBox.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent ie) {}
		});
		gamut1.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				Module mod[] = view3D.getModuleManager().getModules();

				dataItems = new float[mod.length][];
				modToItemIndex = new int[mod.length];
				int sources = 0;

				for (int i = 0; i < mod.length; i++) {
					try {
						GamutData g = (GamutData) mod[i];
						float f[] = g.getData();

						if (f != null && f.length > 5) {
							dataItems[sources] = f;
							modToItemIndex[i] = sources;
							sources++;
						} else {
							modToItemIndex[i] = -1;
						}
					} catch (Exception e) {
						modToItemIndex[i] = -1;
					}
				}
				if (sources == 0) {
					SystemManager.displayError(
						"Error",
						"No gamut source available. Please add a gamut module.");
					return;
				}
				JPopupMenu popMenu = new JPopupMenu("Gamut source");

				items = new JMenuItem[sources];
				int current = 0;

				for (int i = 0; i < mod.length; i++) {
					if (modToItemIndex[i] >= 0) {
						items[current++] = new JMenuItem(((GamutData) mod[i]).getFileName(),
								mod[i].getIcon());
					}
				}
				for (int i = 0; i < items.length; i++) {
					popMenu.add(items[i]);
					items[i].addActionListener(new ActionListener() {
						public void actionPerformed(ActionEvent ae) {
							JMenuItem item = (JMenuItem) ae.getSource();

							for (int i = 0; i < items.length; i++) {
								if (items[i] == item) {
									gamutData[0] = dataItems[i];
									gamut1.setText("Gamut A");
								}
							}
						}
					});
				}
				popMenu.show(gamut1, 0, 0);
			}
		});
		gamut2.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				Module mod[] = view3D.getModuleManager().getModules();

				dataItems = new float[mod.length][];
				modToItemIndex = new int[mod.length];
				int sources = 0;

				for (int i = 0; i < mod.length; i++) {
					try {
						GamutData g = (GamutData) mod[i];
						float f[] = g.getData();

						if (f != null && f.length > 5) {
							dataItems[sources] = f;
							modToItemIndex[i] = sources;
							sources++;
						} else {
							modToItemIndex[i] = -1;
						}
					} catch (Exception e) {
						modToItemIndex[i] = -1;
					}
				}
				if (sources == 0) {
					SystemManager.displayError(
						"Error",
						"No gamut source available. Please add a gamut module.");
					return;
				}
				JPopupMenu popMenu = new JPopupMenu("Gamut source");

				items = new JMenuItem[sources];
				int current = 0;

				for (int i = 0; i < mod.length; i++) {
					if (modToItemIndex[i] >= 0) {
						items[current++] = new JMenuItem(((GamutData) mod[i]).getFileName(),
								mod[i].getIcon());
					}
				}
				for (int i = 0; i < items.length; i++) {
					popMenu.add(items[i]);
					items[i].addActionListener(new ActionListener() {
						public void actionPerformed(ActionEvent ae) {
							JMenuItem item = (JMenuItem) ae.getSource();

							for (int i = 0; i < items.length; i++) {
								if (items[i] == item) {
									gamutData[1] = dataItems[i];
									gamut2.setText("Gamut B");
								}
							}
						}
					});
				}
				popMenu.show(gamut2, 0, 0);
			}
		});
		
		JPanel panel1 = new JPanel(new BorderLayout());

		panel1.add(gamut1, BorderLayout.NORTH);
		
		JPanel panel2 = new JPanel(new BorderLayout());

		panel2.add(gamut2, BorderLayout.NORTH);
		panel1.add(panel2, BorderLayout.SOUTH);
		
		JPanel mPanel = new JPanel(new GridLayout(1, 2));

		mPanel.add(new JLabel("Display"));
		mPanel.add(modeBox);
		
		JPanel panel3 = new JPanel(new BorderLayout());

		panel3.add(mPanel, BorderLayout.NORTH);
		panel2.add(panel3, BorderLayout.SOUTH);
		
		JPanel qPanel = new JPanel(new GridLayout(1, 2));

		qPanel.add(new JLabel("Detail"));
		qPanel.add(quantize);
		
		JPanel panel3a = new JPanel(new BorderLayout());

		panel3a.add(qPanel, BorderLayout.NORTH);
		panel3.add(panel3a, BorderLayout.SOUTH);
		gamutVolume = new JLabel("0");
		gamutVolume.setHorizontalAlignment(JLabel.RIGHT);
		JPanel volPanel = new JPanel(new GridLayout(1, 2));

		volPanel.add(new JLabel("Volume"));
		volPanel.add(gamutVolume);
		
		JPanel panel3aa = new JPanel(new BorderLayout());

		panel3aa.add(volPanel, BorderLayout.NORTH);
		panel3a.add(panel3aa, BorderLayout.SOUTH);
		
		gamutCombo = new JComboBox(visualizationNames[2]);
		gamutCombo.setSelectedIndex(1);
		gamutCombo.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent ie) {
				String type = (String) gamutCombo.getSelectedItem();

				visualization3D[2] = (Visualization3D) visualizations[2].elementAt(visualizationNames[2].indexOf(type));
				cSplit.setBottomComponent(visualization3D[2].getParameterPanel());
			}
		});

		gamutCombo1 = new JComboBox(visualizationNames[0]);
		gamutCombo1.setSelectedIndex(3);
		gamutCombo1.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent ie) {
				String type = (String) gamutCombo1.getSelectedItem();

				visualization3D[0] = (Visualization3D) visualizations[0].elementAt(visualizationNames[0].indexOf(type));
				aSplit.setBottomComponent(visualization3D[0].getParameterPanel());
			}
		});
		gamutCombo2 = new JComboBox(visualizationNames[1]);
		gamutCombo2.setSelectedIndex(3);
		gamutCombo2.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent ie) {
				String type = (String) gamutCombo2.getSelectedItem();

				visualization3D[1] = (Visualization3D) visualizations[1].elementAt(visualizationNames[1].indexOf(type));
				bSplit.setBottomComponent(visualization3D[1].getParameterPanel());
			}
		});
		aSplit.setTopComponent(gamutCombo1);
		aSplit.setBottomComponent(visualization3D[0].getParameterPanel());
		bSplit.setTopComponent(gamutCombo2);
		bSplit.setBottomComponent(visualization3D[1].getParameterPanel());
		cSplit.setTopComponent(gamutCombo);
		cSplit.setBottomComponent(visualization3D[2].getParameterPanel());
		
		JPanel upperPanel = new JPanel(new FlowLayout());

		upperPanel.add(panel1);

		jsp.setTopComponent(upperPanel);
		parameterPanel.add(jsp, BorderLayout.CENTER);
	}
	
	/**
	 * Creates n number of BranchGroups to represent the Gamut in 3D.
	 *
	 * @return array of BranchGroup
	 */
	public BranchGroup[] getSceneBranch() {
		gamutVolume.setText("0");
		if (gamutData[0] != null && gamutData[1] != null) {
			float vertex[][] = new float[2][];

			vertex[0] = gamutData[0];
			vertex[1] = ColorSpaceConverter.convert(gamutData[0], labColorSpace3D,
					view3D.getColorSpaceManager().getColorSpace3D());
			ColorSpace3D inpOup[] = new ColorSpace3D[2];

			inpOup[0] = labColorSpace3D;
			inpOup[1] = view3D.getColorSpaceManager().getColorSpace3D();
			visualization3D[0].createGamutObject(gamutData[0].length / 3, vertex,
					view3D.getColorSpaceManager().getColorSpace3D().toSRGB(vertex[1]), inpOup,
					view3D);
			vertex = new float[2][];
			vertex[0] = gamutData[1];
			vertex[1] = ColorSpaceConverter.convert(gamutData[1], labColorSpace3D,
					view3D.getColorSpaceManager().getColorSpace3D());
			inpOup = new ColorSpace3D[2];
			inpOup[0] = labColorSpace3D;
			inpOup[1] = view3D.getColorSpaceManager().getColorSpace3D();
			visualization3D[1].createGamutObject(gamutData[1].length / 3, vertex,
					view3D.getColorSpaceManager().getColorSpace3D().toSRGB(vertex[1]), inpOup,
					view3D);
			
			int detail = Integer.parseInt(quantize.getSelectedItem().toString());
			BitSet a = visualization3D[0].getVoxel(detail);
			BitSet b = visualization3D[1].getVoxel(detail);
			
			if (modeBox.getSelectedIndex() == 0) {}
			if (modeBox.getSelectedIndex() == 1) {
				a = b;
			}
			if (modeBox.getSelectedIndex() == 2) {
				a.or(b);
			}
			if (modeBox.getSelectedIndex() == 3) {
				a.and(b);
			}
			if (modeBox.getSelectedIndex() == 4) {
				a.andNot(b);
			}
			if (modeBox.getSelectedIndex() == 5) {
				b.andNot(a);
				a = b;
			}
			if (modeBox.getSelectedIndex() == 6) {
				a.xor(b);
			}
				
			int ccc = 0;

			for (int i = 0; i < a.length(); i++) {
				if (a.get(i)) {
					ccc++;
				}
			}
			
			if (ccc < 1) {
				return (new BranchGroup[] {new BranchGroup()});
			}
			
			float adjustment = (1.0f / (float) detail) / 2.0f;
			
			float pointCoord[][] = new float[2][ccc * 3];
			float scale[] = inpOup[1].getScale();
			float dotVolume = (scale[0] / (float) detail) * (scale[1] / (float) detail)
					* (scale[2] / (float) detail);
			float volume = 0;
			int dIndex = 0;

			for (int i = 0; i < detail; i++) {
				for (int j = 0; j < detail; j++) {
					for (int k = 0; k < detail; k++) {
						if (a.get(i * detail * detail + j * detail + k)) {
							volume += dotVolume;
							pointCoord[1][dIndex * 3] = pointCoord[0][dIndex * 3] = (float) i
											/ (float) detail
									+ adjustment;
							pointCoord[1][dIndex * 3 + 1] = pointCoord[0][dIndex * 3 + 1] = (float) j
											/ (float) detail
									+ adjustment;
							pointCoord[1][dIndex * 3 + 2] = pointCoord[0][dIndex * 3 + 2] = (float) k
											/ (float) detail
									+ adjustment;
							dIndex++;
						}
					}
				}
			}
			gamutVolume.setText(Float.toString(volume));
			byte pointColor[] = view3D.getColorSpaceManager().getColorSpace3D().toSRGB(pointCoord[0]);

			return visualization3D[2].createGamutObject(ccc, pointCoord, pointColor,
					new ColorSpace3D[] {view3D.getColorSpaceManager().getColorSpace3D(),
				view3D.getColorSpaceManager().getColorSpace3D()}, view3D);	
		}
		return (new BranchGroup[] {new BranchGroup()});
	}
	
	/**
	 * Returns the parameter panel.
	 * @return The parameter panel.
	 */
	public Component getParameterPanel() {
		if (parameterPanel == null) {
			createParameterPanel();
		}
		return parameterPanel;
	}
	
	/**
	 * Returns the priority of this module.
	 * @return The priority of this module.
	 */
	public int getPriority() {
		return 100;
	}
	
	/**
	 * Returns the name of this module type.
	 * @return The name of the module type.
	 */
	public String getName() {
		return "Gamut comparison";
	}
	
	/**
	 * Determines if one instance of this class should be added to all 3D
	 * scenes as default.
	 * @return True if one instance of this class should be added to all 3D
	 * scenes as default.
	 */
	public boolean existsAsDefault() {
		return false;
	}
	
	/**
	 * Determines if the user should be able to add modules of this type.
	 * @return True if the user should be able to add modules of this type.
	 */
	public boolean userAdd() {
		return true;
	}
	
	/**
	 * Determines if the user should be able to remove modules of this type.
	 * @return True if the user should be able to remove modules of this type.
	 */
	public boolean userRemove() {
		return true;
	}
	
	/**
	 * Returns the icon that represents this module type.
	 * @return The icon.
	 */
	public Icon getIcon() {
		return IconManager.getIcon("gamutscompared");
	}
	
	/**
	 * Deinitializes the module.
	 */
	public void deInitialize() {
		this.gamut1 = null;
		this.gamut2 = null;
		this.gamutCombo = null;
		this.gamutCombo1 = null;
		this.gamutCombo2 = null;
		this.gamutData = null;
		this.jsp = null;
		this.labColorSpace3D = null;
		this.modeBox = null;
		this.parameterPanel = null;
		this.quantize = null;
		this.view3D = null;
		this.visualization3D = null;
		this.visualizationNames = null;
		Visualization3D.deInitialize(visualizations[0]);
		Visualization3D.deInitialize(visualizations[1]);
		Visualization3D.deInitialize(visualizations[2]);
		this.visualizations = null;
	}
	
	/**
	 * Initializes the module.
	 * @param view3D The 3D view where this module is used.
	 */
	public boolean initialize(View3D view3D) {
		this.view3D = view3D;
		labColorSpace3D = ColorSpaceManager.getColorSpace("Lab");
		gamutData = new float[2][];
		visualizationNames = new Vector[3];
		visualizationNames[0] = new Vector();
		visualizationNames[1] = new Vector();
		visualizationNames[2] = new Vector();
		visualizations = new Vector[3];
		visualizations[0] = new Vector();
		visualizations[1] = new Vector();
		visualizations[2] = new Vector();
		String names[] = VisualizationManager.getNames();

		for (int i = 0; i < names.length; i++) {
			visualizationNames[0].add(names[i]);
			visualizationNames[1].add(names[i]);
			visualizationNames[2].add(names[i]);
			visualizations[0].add(VisualizationManager.getVisualization(names[i]));
			visualizations[1].add(VisualizationManager.getVisualization(names[i]));
			visualizations[2].add(VisualizationManager.getVisualization(names[i]));
		}
		visualization3D = new Visualization3D[3];
		visualization3D[0] = (Visualization3D) visualizations[0].elementAt(3);
		visualization3D[1] = (Visualization3D) visualizations[1].elementAt(3);
		visualization3D[2] = (Visualization3D) visualizations[2].elementAt(1);
		visualization3D[0].getParameterPanel();
		visualization3D[1].getParameterPanel();
		
		return true;
	}
	
	/**
	 * Checks whether an update of the 3D scene is possible at this time.
	 * @return True if this module is ready for a 3D update, or false if
	 * for instance an error in the parameter panel prevents an update at
	 * this time.
	 */
	public boolean readyForUpdate() {
		return (visualization3D[0].checkParams() & visualization3D[1].checkParams()
				& visualization3D[2].checkParams());
	}
}
