/*
 ICC3D - Interactive Color Correction in 3 Dimensions.
 Copyright (C) 2002-2003 Gjvik University College.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package icc3dpackage.modules.capture;


import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.*;
import java.util.*;
import javax.media.*;
import javax.media.control.*;
import javax.media.datasink.*;
import javax.media.format.VideoFormat;
import javax.media.protocol.*;


/**
 * Creates a data source/buffer for the movie encoder
 */
public class ImageDataSource extends PullBufferDataSource {
	
	/** The image source streams */
	private ImageSourceStream streams[];
	
	/**
	 * Constructor that initializes the object
	 */
	public ImageDataSource() {
		streams = new ImageSourceStream[1];
		streams[0] = new ImageSourceStream();
	}

	/**
	 * Prepares the stream by setting the size and framerate
	 * @param width the width
	 * @param height the height
	 * @param frameRate the framerate
	 */
	public void prepare(int width, int height, float frameRate) {
		streams[0].prepare(width, height, frameRate);
	}

	/**
	 * Sets the media locator (not used, empty function)
	 * @param source the new media locator
	 */
	public void setLocator(MediaLocator source) {}
	
	/**
	 * Returns the meida locator, in this case always null
	 * @return MediaLocator the media locator
	 */
	public MediaLocator getLocator() {
		return null;
	}

	/**
	 * Returns the content type
	 * @return String the content type
	 */
	public String getContentType() {
		return ContentDescriptor.RAW;
	}

	/**
	 * Not used, included due to inheritance
	 */
	public void connect() {}

	/**
	 * Not used, included due to inheritance
	 */
	public void disconnect() {}

	/**
	 * Not used, included due to inheritance
	 */
	public void start() {}

	/**
	 * Not used, included due to inheritance
	 */
	public void stop() {}

	/**
	 * Returns the streams
	 * @return PullBufferStream[] the streams
	 */
	public PullBufferStream[] getStreams() {
		return streams;
	}

	/**
	 * Returns the time duration
	 * @return Time duration
	 */
	public Time getDuration() {
		return DURATION_UNKNOWN;
	}

	/**
	 * Returns the controls
	 * @return Object[] the controls
	 */
	public Object[] getControls() {
		return new Object[0];
	}
	
	/**
	 * Returns the control given the type
	 * @return Object the control, in this case always null
	 */
	public Object getControl(String type) {
		return null;
	}

	/**
	 * Adds an image to the stream
	 *	@param bi the next image
	 */	
	public void addImage(BufferedImage bi) {
		streams[0].addImage(bi);
	}
	
	/**
	 * Sets the signal no more data (end of input data)
	 */
	public void signalNoMoreData() {
		streams[0].signalNoMoreData();
	}
}
