/*
 ICC3D - Interactive Color Correction in 3 Dimensions.
 Copyright (C) 2002-2003 Gjvik University College.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package icc3dpackage.modules.capture;


import java.awt.*;
import java.util.*;
import javax.swing.*;

import icc3dpackage.managers.*;


/**
 * Parses our simple script language for making movies, and creates
 * MovieInterpolator objects (that actually performes the wanted movement)
 */
public abstract class ScriptParser {

	/**
	 * Constructor that initializes the object
	 */
	public ScriptParser() {}		
	
	/**
	 * Static function that parses the string and creates MovieInterpolator objects
	 * @param parseString string to parse
	 */ 
	public static Vector parse(String parseString) {
		Vector returnVector = new Vector();
		MovieInterpolator current = new MovieInterpolator();

		String text, tmp;

		if (parseString != null && !parseString.equals("")) {
			parseString = parseString.toLowerCase();
			Vector tokens = scanner(parseString, "\n");

			text = (String) tokens.remove(0);
			while (text != null) {	
				if (text.startsWith("move")) {
					StringTokenizer token = new StringTokenizer(text, " ");

					try {
						token.nextToken();
						float xtrans = Float.parseFloat(token.nextToken());
						float ytrans = Float.parseFloat(token.nextToken());		
						float ztrans = Float.parseFloat(token.nextToken());

						current.setXTrans(xtrans);
						current.setYTrans(ytrans);
						current.setZTrans(ztrans);
					} catch (Exception e) {
						SystemManager.displayError(
							"Error",
							"Parse error:\n" + text);
						break;
					}				
				} else if (text.startsWith("rotate")) {
					StringTokenizer token = new StringTokenizer(text, " ");

					try {
						token.nextToken();
						float xrot = Float.parseFloat(token.nextToken());
						float yrot = Float.parseFloat(token.nextToken());		
						float zrot = Float.parseFloat(token.nextToken());

						current.setXRot(xrot);
						current.setYRot(yrot);
						current.setZRot(zrot);
					} catch (Exception e) {
						SystemManager.displayError(
							"Error",
							"Parse error:\n" + text);
						break;
					}
				} else if (text.startsWith("next")) {
					returnVector.add(current);
					current = null;
					current = new MovieInterpolator();
				} else if (text.startsWith("frames")) {
					int frm;

					try {
						frm = Integer.parseInt(text.substring(7, text.length()));
						current.setFrames(frm);
					} catch (Exception e) {
						SystemManager.displayError(
							"Error",
							"Parse error:\n" + text);
						break;
					}
				} else if (text.startsWith("go")) {
					returnVector.add(current);
					break;	
				} else {
					SystemManager.displayError(
						"Error",
						"Parse error:\n" + text);
					break;
				}
  			
				if (tokens.size() > 0) {
					text = (String) tokens.remove(0);
				} else {
					text = null;
				}
			}
		}
		return returnVector;
	}

	/**
	 * Static function that scans a string and returns a vector of tokens
	 * @param scanString string to scan
	 * @param delim delim character
	 * @return Vector vector of tokens, given the delim character
	 */
	public static Vector scanner(String scanString, String delim) {
		Vector returnVector = new Vector();
		StringTokenizer st = new StringTokenizer(scanString, delim, false);

		while (st.hasMoreTokens()) {
			returnVector.add(st.nextToken());
		}
		return returnVector;
	}
}

