/*
 ICC3D - Interactive Color Correction in 3 Dimensions.
 Copyright (C) 2002-2003 Gjvik University College.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package icc3dpackage.modules.regression;


import java.io.BufferedReader;
import java.util.ArrayList;


/**
 * A class that reads color data from a comma separated file.
 */
public abstract class ColorReader {
	
	/**
	 * Reads data from the given file, applying the previously found scale.
	 * @param br The BufferedReader file reader.
	 * @param scale The color space scale.
	 * @return Floating point color data, or null.
	 */
	public static float[][] read(BufferedReader br, float scale[]) {
		// float data[][] = new float[count][6];
		
		ArrayList al = new ArrayList();
		
		int dataRead = 0;
		boolean error = false;
		StringBuffer sb = new StringBuffer();
		StringBuffer floatString = new StringBuffer(50);

		try {
			while (true) {
				String s = br.readLine();

				if (s == null) {
					break;
				}
				sb.append(" ");
				sb.append(s);
			}
		} catch (Exception e) {}
		if (sb.length() > 0) {
			int startIndex, stopIndex;
			int lastStop = -1;

			while (!error) {
				try {
					startIndex = -1;
					stopIndex = -1;
					int i = lastStop + 1;

					while (startIndex == -1) {
						char c = sb.charAt(i);

						if (c >= '0' && c <= '9') {
							startIndex = i;
							break;
						}
						i++;
					}
					i = startIndex;
					while (stopIndex == -1) {
						char c;

						try {
							c = sb.charAt(i);
						} catch (Exception e) {
							c = ' ';
						}
						if (!((c >= '0' && c <= '9') || (c == '.'))) {
							stopIndex = i;
							lastStop = stopIndex;
						}
						i++;
					}
					String s = sb.substring(startIndex, stopIndex);
					float f = Float.parseFloat(s);

					// data[dataRead/6][dataRead%6] = f/scale[dataRead%6];
					al.add(new Float(f));
					dataRead++;
				} catch (Exception e) {
					error = true;
				}
			}
			if (dataRead >= 6) {
				float data[][] = new float[dataRead / 6][6];

				for (int i1 = 0; i1 < dataRead / 6; i1++) {
					for (int i2 = 0; i2 < 6; i2++) {
						data[i1][i2] = ((Float) al.get(i1 * 6 + i2)).floatValue() / scale[i2];
					}
				}
				return data;
			}
		} else {
			return null;
		}
		return null;
	}
}
