/*
 ICC3D - Interactive Color Correction in 3 Dimensions.
 Copyright (C) 2002-2003 Gjvik University College.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package icc3dpackage.modules.regression;


import javax.swing.table.AbstractTableModel;


/**
 * A custom table model.
 */
public class CustomTableModel extends AbstractTableModel {
	
	/** The number of table rows */
	private int rows;
	
	/** The number of table columns*/
	private int columns;
	
	/** The table data */
	private float data[][];
	
	/** The table column names */
	private String columnNames[];
	
	/**
	 * Creates a custom table model with default parameters.
	 */
	public CustomTableModel() {
		columnNames = new String[0];
		rows = 0;
		columns = 0;
		data = new float[0][0];
	}
	
	/**
	 * Creates a custom table model.
	 * @param cells The table data.
	 * @param columnHeaders The table column headers.
	 */
	public CustomTableModel(float cells[][], String columnHeaders[]) {
		data = cells;
		rows = data.length;
		columnNames = columnHeaders;
		if (rows > 0) {
			columns = data[0].length;
		} else {
			columns = 0;
		}
	}
	
	/**
	 * Returns the number of table rows.
	 * @return The number of table rows.
	 */
	public int getRowCount() {
		return rows;
	}
	
	/**
	 * Returns the number of table columns.
	 * @return The number of table columns.
	 */
	public int getColumnCount() {
		return columns;
	}
  
	/**
	 * Returns the value of the table cell at the specified coordinates.
	 * @param row The row number.
	 * @param column The column number.
	 * @return The value of the table cell at the specified coordinates.
	 */
	public Object getValueAt(int row, int column) {
		return Float.toString(data[row][column]);
	}
  
	/**
	 * Returns the value of the table cell at the specified coordinates.
	 * @param row The row number.
	 * @param column The column number.
	 * @return The value of the table cell at the specified coordinates.
	 */
	public float getFloatAt(int row, int column) {
		return data[row][column];
	}
  
	/**
	 * Returns the table data.
	 * @return The table data.
	 */
	public float[][] getData() {
		return data;
	}
  
	/**
	 * Returns the name of the column.
	 * @param column The column.
	 * @return The name of the column.
	 */
	public String getColumnName(int column) {
		return columnNames[column];
	}
  
}
