/*
 ICC3D - Interactive Color Correction in 3 Dimensions.
 Copyright (C) 2002-2003 Gjvik University College.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package icc3dpackage.modules.regression;


import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import javax.swing.border.BevelBorder;

import icc3dpackage.ICC3D;
import icc3dpackage.colorspaces.generic.ColorSpace3D;
import icc3dpackage.managers.SystemManager;
import icc3dpackage.modules.ColorSpaceManager;


/**
 * Displays a color regression frame.
 */
public class RegressionFrame extends JFrame {
	
	/** The CIEXYZ colors */
	private JTable xyzTable;
	
	/** The device specific RGB colors */
	private JTable deviceTable;
	
	/** The CIEXYZ table model */
	private CustomTableModel xyzModel;
	
	/** The device specific table model */
	private CustomTableModel deviceModel;
	
	/** The polynomial degree */
	private JComboBox order;
	
	/** Allow rgeression constants */
	private JCheckBox allowConstants;
	
	/** Reverse polynomial regression */
	private PolynomialRegression reversePR1, reversePR2, reversePR3;
	
	/** Forward polynomial regression */
	private PolynomialRegression forwardPR1, forwardPR2, forwardPR3;
	
	/** Regression performed and result found */
	private boolean ok;
	
	/**
	 * Returns whether the polynomial regression was successfull.
	 * @return True if everything worked.
	 */
	public boolean dataGood() {
		return ok;
	}
	
	/**
	 * Creates a RegressionFrame.
	 */
	public RegressionFrame() {
		super("Regression analysis");
		setIconImage(ICC3D.image);
		float data[][];
		
		try {
			ok = true;
			JFileChooser chooser = new JFileChooser(".");
			int choice = chooser.showOpenDialog(ICC3D.getDialogParent());

			if (choice == JFileChooser.APPROVE_OPTION
					&& chooser.getSelectedFile() != null && chooser.getSelectedFile().isFile()) {
				data = ColorReader.read(new BufferedReader(new java.io.FileReader(chooser.getSelectedFile().getAbsolutePath())),
						new float[] {255, 255, 255, 100, 100, 100});
				if (data == null) {
					data = new float[0][0];
					ok = false;
				}
			} else {
				data = new float[0][0];
				ok = false;
			}
		} catch (Exception e) {
			data = new float[0][0];
			e.printStackTrace();
			ok = false;
		}
		if (!ok) {
			dispose();
			return;
		}
		float data1[][] = new float[data.length][3];
		float data2[][] = new float[data.length][3];
		
		for (int i = 0; i < data.length; i++) {
			data1[i][0] = data[i][0];
			data1[i][1] = data[i][1];
			data1[i][2] = data[i][2];
			data2[i][0] = data[i][3];
			data2[i][1] = data[i][4];
			data2[i][2] = data[i][5];
		}
		xyzModel = new CustomTableModel(data2, new String[] {"X", "Y", "Z"});
		deviceModel = new CustomTableModel(data1, new String[] {"R", "G", "B"});
		xyzTable = new JTable(xyzModel);
		deviceTable = new JTable(deviceModel);
		JPanel tablePanel = new JPanel(new GridLayout(0, 2));
		JPanel xyzPanel = new JPanel(new BorderLayout());

		xyzPanel.setBorder(BorderFactory.createBevelBorder(BevelBorder.RAISED));
		JPanel devicePanel = new JPanel(new BorderLayout());

		devicePanel.setBorder(BorderFactory.createBevelBorder(BevelBorder.RAISED));
		devicePanel.add(deviceTable.getTableHeader(), BorderLayout.NORTH);
		devicePanel.add(deviceTable, BorderLayout.CENTER);
		xyzPanel.add(xyzTable.getTableHeader(), BorderLayout.NORTH);
		xyzPanel.add(xyzTable, BorderLayout.CENTER);
		tablePanel.add(xyzPanel);
		tablePanel.add(devicePanel);
		Container c = getContentPane();

		c.setLayout(new BorderLayout());
		c.add(new JScrollPane(tablePanel), BorderLayout.CENTER);
		
		JPanel buttonPanel = new JPanel(new GridLayout(1, 0));

		allowConstants = new JCheckBox("Allow constants");
		allowConstants.setSelected(true);
		order = new JComboBox(new String[] {	"1st order polynomial",
			"2nd order polynomial", "3rd order polynomial", "4th order polynomial",
			"5th order polynomial", "6th order polynomial", "7th order polynomial",
			"8th order polynomial", "9th order polynomial"});
		JButton calculate = new JButton("Perform analysis");

		calculate.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				int sam = xyzModel.getData().length;
				int var = 3;
				int deg = order.getSelectedIndex() + 1;
				boolean aC = allowConstants.isSelected();
				double f1[] = new double[sam];
				double f2[] = new double[sam];
				double f3[] = new double[sam];
				double f4[] = new double[sam];
				double f5[] = new double[sam];
				double f6[] = new double[sam];
				double v[][] = new double[sam][var];
				double v2[][] = new double[sam][var];

				for (int i = 0; i < sam; i++) {
					f1[i] = xyzModel.getData()[i][0];
					f2[i] = xyzModel.getData()[i][1];
					f3[i] = xyzModel.getData()[i][2];
					f4[i] = deviceModel.getData()[i][0];
					f5[i] = deviceModel.getData()[i][1];
					f6[i] = deviceModel.getData()[i][2];
					v[i][0] = deviceModel.getData()[i][0];
					v[i][1] = deviceModel.getData()[i][1];
					v[i][2] = deviceModel.getData()[i][2];
					v2[i][0] = xyzModel.getData()[i][0];
					v2[i][1] = xyzModel.getData()[i][1];
					v2[i][2] = xyzModel.getData()[i][2];
				}
				PolynomialRegression pr1 = new PolynomialRegression(deg, var, sam, aC);

				pr1.populateMY(f1, v);
				PolynomialRegression pr2 = new PolynomialRegression(deg, var, sam, aC);

				pr2.populateMY(f2, v);
				PolynomialRegression pr3 = new PolynomialRegression(deg, var, sam, aC);

				pr3.populateMY(f3, v);
				PolynomialRegression pr4 = new PolynomialRegression(deg, var, sam, aC);

				pr4.populateMY(f4, v2);
				PolynomialRegression pr5 = new PolynomialRegression(deg, var, sam, aC);

				pr5.populateMY(f5, v2);
				PolynomialRegression pr6 = new PolynomialRegression(deg, var, sam, aC);

				pr6.populateMY(f6, v2);
				boolean success = true;
				double a1[] = null, a2[] = null, a3[] = null, a4[] = null, a5[] = null, a6[] = null;

				try {
					a1 = pr1.getA();
					a2 = pr2.getA();
					a3 = pr3.getA();
					a4 = pr4.getA();
					a5 = pr5.getA();
					a6 = pr6.getA();
				} catch (Exception e) {
					success = false;
					e.printStackTrace();
				}
				if (success) {
					SystemManager.displayError(
						"Analysis completed",
						"Analysis of data completed successfully");
					reversePR1 = pr1;
					reversePR2 = pr2;
					reversePR3 = pr3;
					forwardPR1 = pr4;
					forwardPR2 = pr5;
					forwardPR3 = pr6;
				} else {
					SystemManager.displayError(
						"Error",
						"Analysis failed");
				}
			}
		});
		
		JButton makeSpace = new JButton("Create color space");

		makeSpace.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				if (reversePR1 != null) {
					try {
						File f = new File("prColorSpace");
						String tmp = JOptionPane.showInputDialog(ICC3D.getDialogParent(),
							"Please give the color space a name.\nThe file will be created in\n" +
								f.getAbsolutePath(),
							"colorspace.reg");
						if (tmp == null)
							return;
						BufferedWriter bw = new BufferedWriter(new java.io.FileWriter("prColorSpace" + File.separator + tmp));
						boolean constants = reversePR1.getAllowConstants();
						double a[];
						int order = reversePR1.getDegree();

						/* int point;
						 int detail = 40;
						 double devCoord[] = new double[3];
						 double outData[][] = new double[detail*detail*detail][3];
						 double inData[][] = new double[3][detail*detail*detail];
						 for (int i=0; i<detail; i++) {
						 for (int j=0; j<detail; j++) {
						 for (int k=0; k<detail; k++) {
						 point = i*detail*detail+j*detail+k;
						 devCoord[0] = i/(double)(detail-1);
						 devCoord[1] = j/(double)(detail-1);
						 devCoord[2] = k/(double)(detail-1);
						 inData[0][point] = devCoord[0];
						 inData[1][point] = devCoord[1];
						 inData[2][point] = devCoord[2];
						 outData[point][0] = reversePR1.getValueAt(devCoord);
						 outData[point][1] = reversePR2.getValueAt(devCoord);
						 outData[point][2] = reversePR3.getValueAt(devCoord);
						 }
						 }
						 }
						 
						 
						 PolynomialRegression fwd1 = new PolynomialRegression(order, 3, detail*detail*detail, constants);
						 fwd1.populateMY(inData[0], outData);
						 fwd1.getA();
						 
						 PolynomialRegression fwd2 = new PolynomialRegression(order, 3, detail*detail*detail, constants);
						 fwd2.populateMY(inData[1], outData);
						 fwd2.getA();
						 
						 PolynomialRegression fwd3 = new PolynomialRegression(order, 3, detail*detail*detail, constants);
						 fwd3.populateMY(inData[2], outData);
						 fwd3.getA();
						 */
						bw.write("" + order);
						bw.newLine();
						if (constants == false) {
							bw.write("0");
						}
						a = reversePR1.getOldA();
						for (int i = 0; i < a.length; i++) {
							bw.write("" + a[i]);
							bw.newLine();
						}
						if (constants == false) {
							bw.write("0");
						}
						a = reversePR2.getOldA();
						for (int i = 0; i < a.length; i++) {
							bw.write("" + a[i]);
							bw.newLine();
						}
						if (constants == false) {
							bw.write("0");
						}
						a = reversePR3.getOldA();
						for (int i = 0; i < a.length; i++) {
							bw.write("" + a[i]);
							bw.newLine();
						}
						if (constants == false) {
							bw.write("0");
						}
						a = forwardPR1.getOldA();
						for (int i = 0; i < a.length; i++) {
							bw.write("" + a[i]);
							bw.newLine();
						}
						if (constants == false) {
							bw.write("0");
						}
						a = forwardPR2.getOldA();
						for (int i = 0; i < a.length; i++) {
							bw.write("" + a[i]);
							bw.newLine();
						}
						if (constants == false) {
							bw.write("0");
						}
						a = forwardPR3.getOldA();
						for (int i = 0; i < a.length; i++) {
							bw.write("" + a[i]);
							bw.newLine();
						}
						bw.flush();
						bw.close();
					} catch (Exception e) {
						SystemManager.displayError(
							"Error",
							"Unable to create color space");
					}
				} else {
					SystemManager.displayError(
						"Error",
						"Please perform analysis before creating a color space.");
				}
			}
		});
		buttonPanel.add(order);
		buttonPanel.add(allowConstants);
		buttonPanel.add(calculate);
		buttonPanel.add(makeSpace);
		c.add(buttonPanel, BorderLayout.SOUTH);
		setSize(600, 600);
		show();
	}
	
	/**
	 * Returns the CIEXYZ color data.
	 * @return The CIEXYZ color data.
	 */
	public float[] getXYZ_Data() {
		if (reversePR1 == null || reversePR2 == null || reversePR3 == null) {
			return null;
		}
		int detail = 40;
		int point;
		double devCoord[] = new double[3];
		float data[] = new float[detail * detail * detail * 3];

		for (int i = 0; i < detail; i++) {
			for (int j = 0; j < detail; j++) {
				for (int k = 0; k < detail; k++) {
					point = i * detail * detail + j * detail + k;
					devCoord[0] = i / (double) (detail - 1);
					devCoord[1] = j / (double) (detail - 1);
					devCoord[2] = k / (double) (detail - 1);
					data[point * 3] = (float) reversePR1.getValueAt(devCoord);
					data[point * 3 + 1] = (float) reversePR2.getValueAt(devCoord);
					data[point * 3 + 2] = (float) reversePR3.getValueAt(devCoord);
				}
			}
		}
		return data;
	}
	
	/**
	 * Returns the second set of CIEXYZ color data.
	 * @return The second set of CIEXYZ color data.
	 */
	public float[] getXYZ_Data2() {
		if (reversePR1 == null || reversePR2 == null || reversePR3 == null) {
			return null;
		}
		int detail = 40;
		// int point;
		ColorSpace3D cs3D = ColorSpaceManager.getColorSpace("sRGB");
		float devCoord[] = new float[3];
		float data[] = new float[detail * 3 * 4];

		for (int j = 0; j < 2; j++) {
			for (int k = 0; k < 2; k++) {
				for (int i = 0; i < detail; i++) {
					devCoord[0] = i / (float) (detail - 1);
					devCoord[1] = k;
					devCoord[2] = j;
					devCoord = cs3D.toCIEXYZ(devCoord);
					data[(i + j * detail + k * 2 * detail) * 3] = devCoord[0];
					data[(i + j * detail + k * 2 * detail) * 3 + 1] = devCoord[1];
					data[(i + j * detail + k * 2 * detail) * 3 + 2] = devCoord[2];
				}
			}
		}
		return data;
	}
	
	/**
	 * Returns the set of CIEXYZ color data from the input file.
	 * @return The set of CIEXYZ color data from the input file.
	 */
	public float[] getXYZ_DataFromFile() {
		if (xyzModel == null) {
			return null;
		}
		float t[][] = xyzModel.getData();

		if (t == null) {
			return null;
		}
		float retData[] = new float[t.length * 3];

		for (int i = 0; i < t.length; i++) {
			retData[i * 3] = t[i][0];
			retData[i * 3 + 1] = t[i][1];
			retData[i * 3 + 2] = t[i][2];
		}
		return retData;
	}
}
