/*
 ICC3D - Interactive Color Correction in 3 Dimensions.
 Copyright (C) 2002-2003 Gjvik University College.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package icc3dpackage.parser;


import java.io.*;
import java.util.StringTokenizer;
import javax.swing.JOptionPane;

import icc3dpackage.managers.*;


/**
 * This class is used to parse a IT8.7/2 file, and retrieve Lab-values.
 */
public class Parser {
	
	/**
	 * A BufferedReader used to read the file.
	 */
	private BufferedReader in = null;
	
	/**
	 * Lab index number.
	 */
	private int indexLab_L, indexLab_A, indexLab_B;
	
	/**
	 * The number of fields.
	 */
	private int nFields = 0;
	
	/**
	 * The number of sets.
	 */
	private int nSets = 0;
	
	/**
	 * The data array.
	 */
	private float array[] = null;
	
	/**
	 * A data text string.
	 */
	private String data = new String();
	
	/**
	 * The name of the file.
	 */
	private String fileName;

	/**
	 * Default constructor.
	 *
	 */
	public Parser(String fileName) {
		in = openFile(fileName);	
	}
	 
	/**
	 * Opens a file for reading.
	 *
	 * @param file name of file to parse
	 * @return buffered inputstream, or null if file not found
	 */	
	private BufferedReader openFile(String file) {
		try {
			BufferedReader br = new BufferedReader(new InputStreamReader(new FileInputStream(file)));

			fileName = file;
			return br;
		} catch (FileNotFoundException f) {
			SystemManager.displayError(
				"Error",
				"File not found");
			return null;
		}
	}
	
	/**
	 * Initiates parsing of a given file.
	 *
	 * @return true if no errors, otherwise false
	 */	
	public boolean parse() {
		if (in != null && parseHeaderData() == true) {
			// System.out.println("Parsing file: "+fileName);
			parseData();
			try {
				in.close();
			} catch (IOException e) {}
			return true;
		} else {
			return false;
		}
	}
	
	/**
	 * Reads the header of a given file.
	 * Also used to determine if file is a valid/correct file.
	 *
	 * @return true if fileheader is valid, otherwise false
	 */	
	private boolean parseHeaderData() {
		boolean headerData = true;
		int index = 0;

		try {
			do {
				if (data.startsWith("NUMBER_OF_FIELDS")) {	//
					data = data.substring(data.indexOf(" ")); //
					nFields = Integer.parseInt(data.trim());	// not really necessary
					data = in.readLine();
					if (data != null) {
						data = data.toUpperCase().trim();
					}
				}																					//
				else if (data.startsWith("BEGIN_DATA_FORMAT")) {
					if (parseBEGIN_DATA_FORMAT() == false) {
						return false;
					}
				} else if (data.startsWith("NUMBER_OF_SETS")) {
					data = data.substring(data.indexOf(" "));
					nSets = Integer.parseInt(data.trim());
					data = in.readLine();
					if (data != null) {
						data = data.toUpperCase().trim();
					}
				} else if (data.startsWith("BEGIN_DATA")) {
					headerData = false;	
				} else {
					data = in.readLine();
					if (data != null) {
						data = data.toUpperCase().trim();
					}
				}
			}while (headerData);	
		} catch (Exception e) {
			SystemManager.displayError(
				"Error",
				"IO error");
			return false;
		}
		return true;
	}
	
	/**
	 * Determines in which position at each line Lab values are located.
	 *
	 */
	private boolean parseBEGIN_DATA_FORMAT() {
		String tmp;
		int index = 0;

		try {
			data = in.readLine();
			if (data != null) {
				data = data.toUpperCase().trim();
			}
			tmp = in.readLine();
			if (tmp != null) {
				tmp = tmp.toUpperCase().trim();
			}
			while (!tmp.startsWith("END_DATA_FORMAT")) {
				data += (" " + tmp);
				tmp = in.readLine();
				if (tmp != null) {
					tmp = tmp.toUpperCase().trim();
				}
			}
		} catch (IOException e) {}
		
		StringTokenizer token = new StringTokenizer(data);

		indexLab_L = indexLab_A = indexLab_B = -1;
		while (token.hasMoreTokens()) {
			data = token.nextToken();
			if (data.equalsIgnoreCase("LAB_L")) {
				indexLab_L = index;
			} else if (data.equalsIgnoreCase("LAB_A")) {
				indexLab_A = index;
			} else if (data.equalsIgnoreCase("LAB_B")) {
				indexLab_B = index;
			}
			index++;
		}
		if (indexLab_L != -1 && indexLab_A != -1 && indexLab_B != -1) {
			return true;
		} else {
			SystemManager.displayError(
				"Error",
				"Color data not found in file.");
			return false;
		}
	}
	
	/**
	 * Building datastructure from file. Reading only Lab values.
	 *
	 */
	private void parseData() {
		StringTokenizer tokens;
		int tokenIndex, arrayIndex = 0, tmpIndex = 0;
		float tmpArray[] = new float[nSets * 3];
		String data;
		int n = 0;

		try {
			data = in.readLine();
			if (data != null) {
				data = data.toUpperCase().trim();
			}
			while (!data.startsWith("END_DATA")) {
				try {
					if (data.charAt(0) != '#') {
						tokens = new StringTokenizer(data);
						tokenIndex = 0;
						while (tokens.hasMoreTokens()) {
							data = tokens.nextToken();
							if (tokenIndex == indexLab_L) {
								tmpArray[tmpIndex] = Float.parseFloat(data);
							} else if (tokenIndex == indexLab_A) {
								tmpArray[tmpIndex + 1] = Float.parseFloat(data);
							} else if (tokenIndex == indexLab_B) {
								tmpArray[tmpIndex + 2] = Float.parseFloat(data);
							}
							tokenIndex++;
						}
						tmpIndex = (++arrayIndex) * 3;
					}
					data = in.readLine();
					if (data != null) {
						data = data.toUpperCase().trim();
					}
				} catch (NumberFormatException e) {
					SystemManager.displayError(
						"Error",
						"Internal parser error");
					tmpArray = null;
					break;
				}
			}
		} catch (IOException ioe) {
			SystemManager.displayError(
				"Error",
				"IO error");
		}
		array = tmpArray;
	}

	/**
	 * Retrieves read Lab-data from file.
	 *
	 * @return Lab data
	 */	
	public float[] getData() {
		return array;	
	}
	
	/**
	 * Retrieves number of sets in file.
	 *
	 * @return sets
	 */	
	public int getLength() {
		return nSets;	
	}
	
	/**
	 * Displays data read from file to standard output.
	 *
	 */	
	public void displayArray() {
		int len = array.length / 3;
		int index;

		System.out.println("ArrayLen: " + len);
		for (int i = 0; i < len; i++) {
			index = i * 3;
			System.out.println(i + ": " + array[index] + ", " + array[index + 1] + ", "
					+ array[index + 2]);
		}
	}
}
