/*
 ICC3D - Interactive Color Correction in 3 Dimensions.
 Copyright (C) 2002-2003 Gjvik University College.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package icc3dpackage.parser;


import java.io.*;
import java.util.StringTokenizer;
import javax.swing.JOptionPane;

import icc3dpackage.managers.*;


/**
 * This class is used to parse a given file and retrive Lab-values.
 * File format is: <p>
 * index   R     G     B      L         A        B<p>
 * 0      255	 255	 255	 93.9300	 0.1500	 -1.9100
 *
 */
public class Parser2 {

	/**
	 * The input file.
	 */
	private RandomAccessFile in = null;
	
	/**
	 * The RGB color data array.
	 */
	private int arrayRGB[] = null;
	
	/**
	 * The Lab color data array.
	 */
	private float arrayLab[] = null;
	
	/**
	 * The name of the file.
	 */
	private String fileName;

	/**
	 * Default constructor.
	 *
	 */
	public Parser2(String fileName) {
		in = openFile(fileName);
		if (in != null) {
			parse();
		}
	}
	 
	/**
	 * Opens a file for reading.
	 *
	 * @param file name of file to parse
	 * @return random access stream
	 */	
	private RandomAccessFile openFile(String file) {
		try {
			RandomAccessFile tmpFile = new RandomAccessFile(file, "r");

			fileName = file;
			return tmpFile;
		} catch (FileNotFoundException f) {
			SystemManager.displayError(
				"Error",
				"File not found");
			return null;
		}
	}
	
	/**
	 * Building datastructure from file.
	 * Reading RGB and Lab values.
	 * 
	 * @return true if no errors, otherwise false
	 */	
	public boolean parse() {
		// System.out.println("parsing...");	
		try {
			in.seek(in.length() - 300);
			String tmp = in.readLine();
			String tmp2 = tmp;

			while (tmp2 != null) {
				tmp = tmp2;
				try {
					tmp2 = in.readLine();
				} catch (IOException ioe) {
					tmp2 = null;
				}
			}
			
			StringTokenizer token = new StringTokenizer(tmp);

			try {
				int len = Integer.parseInt(token.nextToken());

				// System.out.println("Length: "+len);
				arrayRGB = new int[(len + 1) * 3];
				arrayLab = new float[(len + 1) * 3];
				in.seek(0);
				for (int i = 0; i < len + 1; i++) {
					tmp = in.readLine();
					token = new StringTokenizer(tmp);
					token.nextToken();
					arrayRGB[i * 3] = Integer.parseInt(token.nextToken());
					arrayRGB[i * 3 + 1] = Integer.parseInt(token.nextToken());
					arrayRGB[i * 3 + 2] = Integer.parseInt(token.nextToken());
					arrayLab[i * 3] = Float.parseFloat(token.nextToken());
					arrayLab[i * 3 + 1] = Float.parseFloat(token.nextToken());
					arrayLab[i * 3 + 2] = Float.parseFloat(token.nextToken());
				}			
			} catch (NumberFormatException nfe) {
				SystemManager.displayError(
					"Error",
					"Internal parser error. Probably wrong file format");
				return false;
			}
		} catch (IOException ioe) {
			arrayRGB = null;
			arrayLab = null;
			return false;	
		}
		return true;
	}	
	
	/**
	 * Returns RGB values.
	 *
	 * @return RGB values
	 */
	public int[] getRGBData() {
		return arrayRGB;	
	}

	/**
	 * Returns Lab values.
	 *
	 * @return Lab values
	 */
	public float[] getLabData() {
		return arrayLab;	
	}
	
	/**
	 * Display Lab values to standard output
	 *
	 */	
	public void displayLab() {
		if (arrayLab != null) {
			int len = arrayLab.length / 3;

			for (int i = 0; i < len; i++) {
				System.out.println(i + ": " + arrayLab[i * 3] + " , " + arrayLab[i * 3 + 1]
						+ " , " + arrayLab[i * 3 + 2]);
			}
		}
			
	}
}
