/*
 ICC3D - Interactive Color Correction in 3 Dimensions.
 Copyright (C) 2002-2003 Gjvik University College.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package icc3dpackage.parser;


import java.io.*;
import java.util.*;
import javax.swing.JOptionPane;

import icc3dpackage.managers.*;


/**
 * This class is used to parse a text file, and retrieve Lab-values.
 */
public class Parser3 {
	
	/**
	 * A BufferedReader used to read the file.
	 */
	private BufferedReader in = null;
	
	/**
	 * The data array.
	 */
	private float array[];
	
	/**
	 * The name of the file.
	 */
	private String fileName;

	/**
	 * Default constructor.
	 */
	public Parser3(String fileName) {
		in = openFile(fileName);	
	}
	 
	/**
	 * Opens a file for reading.
	 *
	 * @param file name of file to parse
	 * @return buffered inputstream, or null if file not found
	 */	
	private BufferedReader openFile(String file) {
		try {
			BufferedReader br = new BufferedReader(new InputStreamReader(new FileInputStream(file)));

			fileName = file;
			return br;
		} catch (FileNotFoundException f) {
			SystemManager.displayError(
				"Error",
				"File not found");
			return null;
		}
	}
	
	/**
	 * Initiates parsing of a given file.
	 *
	 * @return true if no errors, otherwise false
	 */	
	public boolean parse() {
		boolean success = false;

		if (in != null) {
			boolean b = parseData();

			success = b;
			try {
				in.close();
			} catch (IOException e) {}
			return true;
		}
		return success;
	}
	
	/**
	 * Building datastructure from file. Reading only Lab values.
	 */
	private boolean parseData() {
		try {
			ArrayList al = new ArrayList();

			try {
				while (true) {
					String line = in.readLine();
					StringTokenizer token = new StringTokenizer(line);
					float f[] = new float[3];

					f[0] = Float.parseFloat(token.nextToken());
					f[1] = Float.parseFloat(token.nextToken());
					f[2] = Float.parseFloat(token.nextToken());
					al.add(f);
				}
			} catch (Exception e) {}
			if (al.size() > 0) {
				array = new float[al.size() * 3];
				for (int i = 0; i < al.size(); i++) {
					array[i * 3 + 1] = ((float[]) al.get(i))[0];
					array[i * 3 + 2] = ((float[]) al.get(i))[1];
					array[i * 3] = ((float[]) al.get(i))[2];
				}
				return true;
			} else {
				return false;
			}
		} catch (Exception e) {
			return false;
		}
	}

	/**
	 * Retrieves read Lab-data from file.
	 *
	 * @return Lab data
	 */	
	public float[] getData() {
		return array;	
	}

	/**
	 * Displays data read from file to standard output.
	 *
	 */	
	public void displayArray() {
		int len = array.length / 3;
		int index;

		System.out.println("ArrayLen: " + len);
		for (int i = 0; i < len; i++) {
			index = i * 3;
			System.out.println(i + ": " + array[index] + ", " + array[index + 1] + ", "
					+ array[index + 2]);
		}
	}
}
