/*
 ICC3D - Interactive Color Correction in 3 Dimensions.
 Copyright (C) 2002-2003 Gjvik University College.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package icc3dpackage.pickable;


/**
 * An interface which defines the functions necessary for a 3D object to 
 * have picking abilities.
 */
public interface PickableObject {
	
	/**
	 * Sets the position of the object.
	 * @param pos The position.
	 */
	public void setPosition(float pos[]);
	
	/**
	 * Updates the object position.
	 */
	public void updatePosition();
	
	/**
	 * Updates the object color.
	 */
	public void updateColor();
	
	/**
	 * Returns the object position by reference.
	 */
	public float[] getPositionReference();
	
	/**
	 * Returns the object color by reference.
	 */
	public byte[] getColorReference();
	
	/**
	 * Returns the position of the object.
	 * @return The object position.
	 */
	public float[] getPosition();
	
	/**
	 * Returns the colors that belong to the object.
	 * @return The indices of the colors that belong to the object.
	 */
	public int[] getColorIndices();
	
	/**
	 * Returns the color of this object.
	 * @return The color of this object.
	 */
	public byte[] getColor();
	
	/**
	 * Sets the color of the object.
	 * @param col The color.
	 */
	public void setColor(byte col[]);
} 

