/*
 ICC3D - Interactive Color Correction in 3 Dimensions.
 Copyright (C) 2002-2003 Gjvik University College.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package icc3dpackage.tiffreader.tags;


import icc3dpackage.tiffreader.tags.generic.TIFF_Tag;
import icc3dpackage.tiffreader.IFD_Entry;
import icc3dpackage.utils.RandomAccessFileWrapper;


/**
 * This TIFF tag contains information about extra samples.
 */
public class ExtraSamplesTag extends TIFF_Tag {
	
	/** The tag id. */
	public final static int TAG_ID = 338;
	
	/** The tag description. */
	public final static String TAG_DESCRIPTION = "Extra samples";
	
	/** Unspecified data. */
	public final static int UNSPECIFIED_DATA = 0;
	
	/** Associated alpha channel. */
	public final static int ASSOCIATED_ALPHA_DATA = 1;
	
	/** Unassociated alpha data. */
	public final static int unASSOCIATED_ALPHA_DATA = 2;
	
	/** The data length. */
	private int length;
	
	/** The extra sample types. */
	private int data[];
	
	/**
	 * Returns the tag id.
	 * @return The tag id.
	 */
	public static int getTagID() {
		return TAG_ID;
	}
	
	/**
	 * Returns a description of the tag.
	 * @return A String that describes the tag.
	 */
	public static String getTagDescription() {
		return TAG_DESCRIPTION;
	}
	
	/**
	 * Creates the tag.
	 * @param entry The parent directory.
	 * @param rafw The TIFF file.
	 */
	public ExtraSamplesTag(IFD_Entry entry, RandomAccessFileWrapper rafw) {
		if (entry.getType() != IFD_Entry.TYPE_SHORT) {
			throw new RuntimeException("TIFF extra samples tag doesn't contain short unsigned integers");
		}
		length = (int) entry.getCount();
		data = new int[length];
		rafw.setOffset(entry.getValueOrOffset());
		for (int i = 0; i < length; i++) {
			data[i] = rafw.getInt();
			if (data[i] > 2 || data[i] < 0) {
				throw new RuntimeException("TIFF extra samples tag contains unknown data types");
			}
		}
	}
}
