/*
 ICC3D - Interactive Color Correction in 3 Dimensions.
 Copyright (C) 2002-2003 Gjvik University College.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package icc3dpackage.tiffreader.tags;


import icc3dpackage.tiffreader.tags.generic.TIFF_Tag;
import icc3dpackage.tiffreader.IFD_Entry;
import icc3dpackage.utils.RandomAccessFileWrapper;


/**
 * This TIFF tag contains information about threshholding.
 */
public class ThreshholdingTag extends TIFF_Tag {
	
	/** The tag description. */
	public final static String TAG_DESCRIPTION = "Threshholding";
	
	/** The tag id. */
	public final static int TAG_ID = 263;
	
	/** No dithering applied. */
	public final static int NO_DITHERING = 1;
	
	/** Ordered dithering applied. */
	public final static int ORDERED_DITHERING = 2;
	
	/** Error diffusion applied. */
	public final static int ERROR_DIFFUSION = 3;
	
	/** The threshholding applied. */
	private int thresh;
	
	/**
	 * Returns the tag id.
	 * @return The tag id.
	 */
	public static int getTagID() {
		return TAG_ID;
	}
	
	/**
	 * Returns a description of the tag.
	 * @return A String that describes the tag.
	 */
	public static String getTagDescription() {
		return TAG_DESCRIPTION;
	}
	
	/**
	 * Creates the tag.
	 * @param entry The parent directory.
	 * @param rafw The TIFF file.
	 */
	public ThreshholdingTag(IFD_Entry entry, RandomAccessFileWrapper rafw) {
		if (entry.getType() != IFD_Entry.TYPE_SHORT) {
			throw new RuntimeException("TIFF threshholding tag doesn't contain short/long unsigned integers");
		}
		if (entry.getCount() != 1) {
			throw new RuntimeException("TIFF threshholding tag doesn't contain the correct number of values");
		}
		thresh = entry.getValue1();
	}
	
	/**
	 * Default constructor.
	 */
	public ThreshholdingTag() {
		thresh = 1;
	}
	
	/**
	 * Returns the type of threhholding applied.
	 * @return The type of threhholding applied.
	 */
	public int getThreshholding() {
		return thresh;
	}
	
	/**
	 * Prints information about the tag.
	 */
	public void printInfo() {
		System.out.println("Threshholding: " + thresh);
	}
}
