/*
 ICC3D - Interactive Color Correction in 3 Dimensions.
 Copyright (C) 2002-2003 Gjvik University College.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package icc3dpackage.triangulation;


/**
 * A class that is used to represent the edge between two Facets.
 */
public class Edge {

	/**
	 * The start vertex.
	 */
	Vertex start;
	
	/**
	 * The middle vertex.
	 */
	Vertex middle;
	
	/**
	 * The end vertex.
	 */
	Vertex end;
	
	/**
	 * The Facet on the inside of this edge.
	 */
	Facet inside;
	
	/**
	 * The Facet on the outside of this edge.
	 */
	Facet outside;
	
	/**
	 * Construct an Edge consisting of the specified vertices and facet.
	 * @param start The start vertex.
	 * @param end The start vertex.
	 * @param outside The outside facet.
	 */
	public Edge(Vertex start, Vertex end, Facet outside) {
		this.start = start;
		this.end = end;
		this.outside = outside;
	}
	
	/**
	 * Construct an Edge consisting of the specified vertices and facet.
	 * @param start The start vertex.
	 * @param middle The middle vertex.
	 * @param end The start vertex.
	 * @param outside The outside facet.
	 */
	public Edge(Vertex start, Vertex middle, Vertex end, Facet outside) {
		this.start = start;
		this.middle = middle;
		this.end = end;
		this.outside = outside;
	}
}
