/*
 ICC3D - Interactive Color Correction in 3 Dimensions.
 Copyright (C) 2002-2003 Gjvik University College.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package icc3dpackage.triangulation;


/**
 * A class that represents a hyper plane in n
 * dimensions.
 */
public class HyperPlane {
	
	public static final Vertex zVector = new Vertex(0, 0, 1);
	
	/**
	 * The plane normal.
	 */
	Vertex normal; 
	
	/**
	 * The distance from origo to the plane in the direction of the plane normal.
	 */
	double d;
	
	/**
	 * Constructs a HyperPlane from the given vertices.
	 * @param a Vertex a.
	 * @param b Vertex b.
	 * @param c Vertex c.
	 */
	public HyperPlane(Vertex a, Vertex b, Vertex c) {
		normal = b.subtract(a).cross(c.subtract(a)).normalize();
		d = normal.dot(a);
	}

	/**
	 * Constructs a HyperPlane from the given vertices.
	 * @param a Vertex a.
	 * @param b Vertex b.
	 */
	public HyperPlane(Vertex a, Vertex b) {
		normal = b.subtract(a).cross(zVector).normalize();
		d = normal.dot(a);
	}
	
	/**
	 * Constructs a HyperPlane from the given vertices.
	 * @param a Vertex a.
	 * @param b Vertex b.
	 * @param n - .
	 */
	public HyperPlane(Vertex a, Vertex b, int n) {
		normal = b.subtract(a).cross(zVector).normalize();
		normal = new Vertex(normal.coordinates[0], normal.coordinates[1],
				normal.coordinates[2], 0.0);
		d = normal.dot(a);
	}

	/**
	 * Returns true if the vertex is outside the hyper plane.
	 * @return true if the vertex is outside the hyper plane.
	 */
	public boolean outside(Vertex x) {
		return normal.dot(x) > d;
	}
  
	/**
	 * Reassign and recompute the hyper plane.
	 * @param a Vertex a.
	 * @param b Vertex b.
	 * @param c Vertex c.
	 */
	public void assign(Vertex a, Vertex b, Vertex c) {
		normal = b.subtract(a).cross(c.subtract(a)).normalize();
		d = normal.dot(a);
	}
}
