/*
 ICC3D - Interactive Color Correction in 3 Dimensions.
 Copyright (C) 2002-2003 Gjvik University College.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package icc3dpackage.triangulation;


/**
 * A class that is used to store objects.
 */
public class OptimizedVector {
	
	/**
	 * The objects.
	 */
	public Object[] elementData;
	
	/**
	 * The object count.
	 */
	public int elementCount;

	/**
	 * Default constructor.
	 */
	public OptimizedVector() {
		elementData = new Object[20];
	}
	
	/**
	 * Constructs an OptimizedVector with the specified capacity.
	 * @param initial Initial capacity.
	 */
	public OptimizedVector(int initial) {
		elementData = new Object[initial];
	}
  
	/**
	 * Adds an object to this OptimizedVector
	 * @param e The object.
	 */
	public void addElement(Object e) {
		if (elementData.length == elementCount) {
			Object[] newData = new Object[1 + 2 * elementData.length];

			System.arraycopy(elementData, 0, newData, 0, elementCount);
			elementData = newData;
		}
		elementData[elementCount++] = e;
	}
  
	/**
	 * Appends all object from another OptimizedVector to this OptimizedVector.
	 * @param l The OptimizedVector.
	 */
	public void append(OptimizedVector l) {
		if (elementData.length < elementCount + l.elementCount) {
			Object[] newData = new Object[Math.max(1 + 2 * elementData.length, 1 + elementCount + l.elementCount)];

			System.arraycopy(elementData, 0, newData, 0, elementCount);
			elementData = newData;
		}  	
		for (int i = 0; i < l.elementCount; i++) {
			elementData[elementCount++] = l.elementData[i];
		}
	}

	/**
	 * Returns the object at index i.
	 * @param i The index.
	 */
	public Object elementAt(int i) {
		return elementData[i];
	}
  
	/**
	 * Returns the number of objects.
	 * @return The number of objects currently stored in this OptimizedVector.
	 */
	public int size() {
		return elementCount;
	}

	/**
	 * Removes all objects from this OptimizedVector.
	 */
	public void clear() {
		elementCount = 0;
	}
}
