/*
 ICC3D - Interactive Color Correction in 3 Dimensions.
 Copyright (C) 2002-2003 Gjvik University College.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package icc3dpackage.utils;


import icc3dpackage.colorspaces.generic.*;


/**
 * This abstract class serves as common color space converter for all ColorSpace3D classes.
 */
public abstract class ColorSpaceConverter {

	/**
	 * Converts a color in the specified input color space to the spesified output color space.
	 *
	 * @param color The float array of colors to convert.
	 * @param from The original color space
	 * @param to The desired color space
	 * @return A float array of colors in the desired color space.
	 */
	public static float[] convert(float color[], ColorSpace3D from, ColorSpace3D to) {
		if (from.getName().equals(to.getName())) {
			float retColor[] = new float[color.length];

			System.arraycopy(color, 0, retColor, 0, color.length);
			return retColor;
		}
		return to.fromCIEXYZ(from.toCIEXYZ(color));
	}
} 

