/*
 ICC3D - Interactive Color Correction in 3 Dimensions.
 Copyright (C) 2002-2003 Gjvik University College.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package icc3dpackage.view3d;


import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

import icc3dpackage.*;
import icc3dpackage.managers.SystemManager;


/**
 * Status display for the 3D view.
 */
public class Status3D extends JPanel implements MouseListener {
	
	/** Left status label. */
	private JLabel left;
	
	/** Middle status label. */
	private JLabel middle;
	
	/** Right status label. */
	private JLabel right;
	
	/**
	 * Creates a status bar.
	 */
	public Status3D() {
		setLayout(new BorderLayout());
		setBorder(BorderFactory.createBevelBorder(BevelBorder.LOWERED));
		left = new JLabel("ICC3D " + ICC3D.getVersion() + " ready", JLabel.LEFT);
		middle = new JLabel("", JLabel.CENTER);
		right = new JLabel("", JLabel.RIGHT);
		JPanel innerPanel = new JPanel(new BorderLayout());

		innerPanel.setBorder(BorderFactory.createEmptyBorder(2, 2, 2, 2));
		innerPanel.add(left, BorderLayout.WEST);
		innerPanel.add(middle, BorderLayout.CENTER);
		innerPanel.add(right, BorderLayout.EAST);
		add(innerPanel, BorderLayout.CENTER);
		addMouseListener(this);
	}
	
	/**
	 * Sets the left status label.
	 * @param s The new status text.
	 */
	public void setLeft(String s) {
		left.setText(s);
	}
	
	/**
	 * Sets the middle status label.
	 * @param s The new status text.
	 */
	public void setMiddle(String s) {
		middle.setText(s);
	}
	
	/**
	 * Sets the right status label.
	 * @param s The new status text.
	 */
	public void setRight(String s) {
		right.setText(s);
	}
	
	/**
	 * Sets the color of the status bar.
	 * @param c The color.
	 */
	public void setColor(Color c) {
		left.setBackground(c);
	}
	
	/**
	 * Recalculate memory consumption on mouse click.
	 * @param e The mosue event that triggered the action.
	 */
	public void mouseClicked(MouseEvent e) {
		SystemManager.majorClean();
		setRight("Memory usage: " + SystemManager.getUsedMemory() + " "
				+ (java.text.DecimalFormat.getPercentInstance().format((double) SystemManager.getUsedMemoryInBytes()
						/ (double) SystemManager.getTotalMemory())));
	}
	
	/**
	 * Do nothing.
	 * @param e The mouse event.
	 */
	public void mouseEntered(MouseEvent e) {}
	
	/**
	 * Do nothing.
	 * @param e The mouse event.
	 */
	public void mouseExited(MouseEvent e) {}
	
	/**
	 * Do nothing.
	 * @param e The mouse event.
	 */
	public void mousePressed(MouseEvent e) {}
	
	/**
	 * Do nothing.
	 * @param e The mouse event.
	 */
	public void mouseReleased(MouseEvent e) {}
}
