/*
 ICC3D - Interactive Color Correction in 3 Dimensions.
 Copyright (C) 2002-2003 Gjvik University College.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package icc3dpackage.visualizations;


import java.awt.*;
import javax.media.j3d.*;
import javax.swing.*;
import java.util.BitSet;

import icc3dpackage.*;
import icc3dpackage.modules.ImageModule;
import icc3dpackage.colorspaces.generic.*;
import icc3dpackage.view3d.View3D;
import icc3dpackage.visualizations.generic.*;


/**
 * NoneVisualization3D is a visualization which displays nothing.
 */
public class NoneVisualization3D extends Visualization3D {
	private JPanel parameterPanel;
	
	/**
	 * Default constructor for a NoneVisualization3D.
	 */
	public NoneVisualization3D() {}
	
	/**
	 * Computes and returns a voxel representation of the gamut.
	 *
	 * @param detail The detail level.
	 * @return A BitSet voxel representaion of the gamut. (1=inside, 0= outside).
	 */
	public BitSet getVoxel(int detail) {
		BitSet voxel = new BitSet(detail * detail * detail);

		return voxel;
	}
	
	/**
	 * Computes and returns a scene graph given a set of points and input/output
	 * color spaces. This particular implementation returns an empty scene graph.
	 *
	 * @param numPoints The number of input points.
	 * @param pointCoord The coordinates of the points, in both the input and output color space.
	 *	@param numEqual The number of pixels that share the given vertex coordinates in 3D.
	 * @param pointColor The color of the points.
	 * @param pixelIndex For each point, gives the indices of the image pixels that belong to the point.
	 * @param colorSpace3D The input and output color space.
	 * @return The branch groups that represent the visualization. BranchGroup[0] is the pickable point array,
	 * while BranchGroup[1] is the non-selectable part of the visualization. Both branch groups
	 * may be null.        
	 */ 
	public BranchGroup[] createSceneObject(int numPoints, float pointCoord[][], int numEqual[], byte pointColor[], int pixelIndex[][], ColorSpace3D colorSpace3D[], ImageModule imageManager, View3D view3D) {
		return null;
	}

	/**
	 * Returns the name (short description) of the visualization.
	 * @return The name of the visualization. (None)
	 */	
	public String getName() {
		return "None";
	}
	
	/**
	 * The priority of the visualization.
	 * @return An int representing the priority of the visualization (lower => higher priority).
	 */
	public int getPriority() {
		return 6;
	}
	
	/**
	 * Returns the panel that allows the user to modify visualization parameters.
	 * @return The parameter panel.
	 */
	public Component getParameterPanel() {
		if (parameterPanel == null) {
			createParameterPanel();
		}
		return parameterPanel;
	}
	
	/**
	 * Creates the parameter panel for this visualization.
	 */
	private void createParameterPanel() {
		parameterPanel = new JPanel(new BorderLayout());
		parameterPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createRaisedBevelBorder(),
				"None"));
	}

	/**
	 * Checks that the parameters are within normal range.
	 * @return True if everyting checks out, false if there are errors.
	 */	
	public boolean checkParams() {
		return true;
	}
	
	/**
	 * Releases any memory that is no longer needed.
	 */
	public void clearMemory() {}
	
	/**
	 * Deinitializes the visualization, and frees any memory that is no longer needed.
	 */
	public void deInitialize() {
		this.parameterPanel = null;
	}
}
